Math.randomString = function (n) {
    var text = "";
    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
    for (var i = 0; i < n; i++) {
        text += possible.charAt(Math.floor(Math.random() * possible.length));
    }
    return text;
};

var Lobibox = Lobibox || {};

(function () {
    var LobiboxNotify = function (type, options) {
        this.$type = null;
        this.$options = null;
        this.$el = null;

        var me = this;

        var _processInput = function (options) {
            if (options.size === "mini" || options.size === "large") {
                options = Object.assign({}, Lobibox.notify.OPTIONS[options.size], options);
            }
            options = Object.assign({}, Lobibox.notify.OPTIONS[me.$type], Lobibox.notify.DEFAULTS, options);

            if (options.size !== "mini" && options.title === true) {
                options.title = Lobibox.notify.OPTIONS[me.$type].title;
            } else if (options.size === "mini" && options.title === true) {
                options.title = false;
            }

            if (options.icon === true) {
                options.icon = Lobibox.notify.OPTIONS.icons[options.iconSource][me.$type];
            }

            if (options.sound === true) {
                options.sound = options.soundPath + options.sound + options.soundExt;
            }

            return options;
        };

        var _appendInWrapper = function ($el, $wrapper) {
            if (me.$options.size === "normal" || me.$options.size === "mini") {
                if ($wrapper.classList.contains("bottom")) {
                    $wrapper.insertBefore($el, $wrapper.firstChild);
                } else {
                    $wrapper.appendChild($el);
                }
            } else if (me.$options.size === "large") {
                var tabPane = _createTabPane().appendChild($el);
                var $li = _createTabControl(tabPane.id);
                $wrapper.querySelector(".lb-notify-wrapper").appendChild(tabPane);
                $wrapper.querySelector(".lb-notify-tabs").appendChild($li);
                _activateTab($li);
                $li.querySelector(">a").addEventListener("click", function () {
                    _activateTab($li);
                });
            }
        };

        var _activateTab = function ($li) {
            var tabs = $li.closest(".lb-notify-tabs").querySelectorAll(">li");
            tabs.forEach(function (tab) {
                tab.classList.remove("active");
            });
            $li.classList.add("active");

            var current = document.querySelector($li.querySelector(">a").getAttribute("href"));
            var panes = current.closest(".lb-notify-wrapper").querySelectorAll(">.lb-tab-pane");
            panes.forEach(function (pane) {
                pane.classList.remove("active");
            });
            current.classList.add("active");
        };

        var _createTabControl = function (tabPaneId) {
            var $li = document.createElement("li");
            $li.className = Lobibox.notify.OPTIONS[me.$type]["class"];
            var $a = document.createElement("a");
            $a.setAttribute("href", "#" + tabPaneId);
            $a.innerHTML = '<i class="tab-control-icon ' + me.$options.icon + '"></i>';
            $li.appendChild($a);
            return $li;
        };

        var _createTabPane = function () {
            var $div = document.createElement("div");
            $div.className = "lb-tab-pane";
            $div.id = Math.randomString(10);
            return $div;
        };

        var _createNotifyWrapper = function () {
            var selector = (me.$options.size === "large" ? ".lobibox-notify-wrapper-large" : ".lobibox-notify-wrapper") + "." + me.$options.position.replace(/\s/gi, ".");
            var $wrapper = document.querySelector(selector);
            if (!$wrapper) {
                $wrapper = document.createElement("div");
                $wrapper.className = selector.replace(/\./g, " ").trim();
                document.body.appendChild($wrapper);
                if (me.$options.size === "large") {
                    var ul = document.createElement("ul");
                    ul.className = "lb-notify-tabs";
                    $wrapper.appendChild(ul);
                    var div = document.createElement("div");
                    div.className = "lb-notify-wrapper";
                    $wrapper.appendChild(div);
                }
            }
            return $wrapper;
        };

        var _createNotify = function () {
            var OPTS = Lobibox.notify.OPTIONS;
            var $notify = document.createElement("div");
            $notify.className = "lobibox-notify " + OPTS[me.$type]["class"] + " " + OPTS["class"] + " " + me.$options.showClass;

            var $iconWrapper = document.createElement("div");
            $iconWrapper.className = "lobibox-notify-icon-wrapper";
            $notify.appendChild($iconWrapper);

            var $iconEl = document.createElement("div");
            $iconEl.className = "lobibox-notify-icon";
            $iconWrapper.appendChild($iconEl);

            var $innerIconEl = document.createElement("div");
            $iconEl.appendChild($innerIconEl);

            if (me.$options.img) {
                $innerIconEl.innerHTML = '<img src="' + me.$options.img + '"/>';
            } else if (me.$options.icon) {
                // $innerIconEl.innerHTML = '<div class="icon-el"><i class="' + me.$options.icon + '" style="font-size: 13px;"></i></div>';
            } else {
                $notify.classList.add("without-icon");
            }

            var $msg = document.createElement("div");
            $msg.className = "lobibox-notify-msg";
            $msg.innerHTML = me.$options.msg;
            if (me.$options.messageHeight !== false) {
                $msg.style.maxHeight = me.$options.messageHeight + "px";
            }

            var $body = document.createElement("div");
            $body.className = "lobibox-notify-body";
            $body.appendChild($msg);
            $notify.appendChild($body);

            if (me.$options.title) {
                var $title = document.createElement("div");
                $title.className = "lobibox-notify-title";
                $title.innerHTML = me.$options.title;
                $body.insertBefore($title, $msg);
            }

            _addCloseButton($notify);

            if (me.$options.size === "normal" || me.$options.size === "mini") {
                _addCloseOnClick($notify);
                _addDelay($notify);
            }

            if (me.$options.width) {
                $notify.style.width = _calculateWidth(me.$options.width) + "px";
            }

            return $notify;
        };

        var _addCloseButton = function ($el) {
            if (me.$options.closable) {
                var $closeBtn = document.createElement("span");
                $closeBtn.className = "lobibox-close";
                $closeBtn.innerHTML = "&times;";
                $closeBtn.addEventListener("click", function (ev) {
                    ev.preventDefault();
                    ev.stopPropagation();
                    me.remove();
                });
                $el.appendChild($closeBtn);
            }
        };

        var _addCloseOnClick = function ($el) {
            if (me.$options.closeOnClick) {
                $el.addEventListener("click", function () {
                    me.remove();
                });
            }
        };

        var _addDelay = function ($el) {
            if (me.$options.delay) {
                if (me.$options.delayIndicator) {
                    var delay = document.createElement("div");
                    delay.className = "lobibox-delay-indicator";
                    delay.innerHTML = "<div></div>";
                    $el.appendChild(delay);
                }
                var time = 0;
                var interval = 1000 / 30;
                var currentTime = new Date().getTime();
                var timer = setInterval(function () {
                    if (me.$options.continueDelayOnInactiveTab) {
                        time = new Date().getTime() - currentTime;
                    } else {
                        time += interval;
                    }
                    var width = (100 * time) / me.$options.delay;
                    if (width >= 100) {
                        width = 100;
                        me.remove();
                        clearInterval(timer);
                    }
                    if (me.$options.delayIndicator) {
                        delay.querySelector("div").style.width = width + "%";
                    }
                }, interval);

                if (me.$options.pauseDelayOnHover) {
                    $el.addEventListener("mouseenter", function () {
                        interval = 0;
                    });
                    $el.addEventListener("mouseleave", function () {
                        interval = 1000 / 30;
                    });
                }
            }
        };

        var _findTabToActivate = function ($li) {
            var $itemToActivate = $li.previousElementSibling;
            if (!$itemToActivate) {
                $itemToActivate = $li.nextElementSibling;
            }
            return $itemToActivate || null;
        };

        var _calculateWidth = function (width) {
            return Math.min(window.outerWidth, width);
        };

        this.remove = function () {
            me.$el.classList.remove(me.$options.showClass);
            me.$el.classList.add(me.$options.hideClass);

            if (me.$options.size === "large") {
                var $li = document.querySelector('.lb-notify-tabs li a[href="#' + me.$el.id + '"]').closest("li");
                var $wrapper = $li.closest(".lobibox-notify-wrapper-large");
                var $paneToActivate = _findTabToActivate($li);
                $li.remove();
                me.$el.addEventListener("animationend", function () {
                    me.$el.remove();
                    if ($paneToActivate) {
                        _activateTab($paneToActivate);
                    }
                });
            } else {
                me.$el.addEventListener("animationend", function () {
                    me.$el.remove();
                });
            }
        };

        this.$type = type;
        this.$options = _processInput(options);
        this.$el = _createNotify();
        _appendInWrapper(this.$el, _createNotifyWrapper());

        // if (this.$options.sound) {
        //     var snd = new Audio(this.$options.sound);
        //     snd.play();
        // }
    };

    Lobibox.notify = function (type, options) {
        return new LobiboxNotify(type, options);
    };

    Lobibox.notify.DEFAULTS = {
        title: true,
        size: "normal",
        soundPath: "sounds/",
        soundExt: ".ogg",
        iconSource: "fontAwesome",
        msg: "",
        showClass: "fadeInDown",
        hideClass: "zoomOut",
        delay: 5000,
        delayIndicator: true,
        pauseDelayOnHover: true,
        continueDelayOnInactiveTab: false,
        closable: true,
        closeOnClick: true,
        width: 400,
        messageHeight: 60,
    };

    Lobibox.notify.OPTIONS = {
        "class": "animated-fast",
        large: {
            width: 500,
            messageHeight: 120,
            delayIndicator: false,
        },
        mini: {
            // width: 200,
            messageHeight: 32,
            delayIndicator: false,
        },
        success: {
            "class": "lobibox-notify-success",
            title: "Success",
            icon: true,
            sound: true,
        },
        error: {
            "class": "lobibox-notify-error",
            title: "Error",
            icon: true,
            sound: true,
        },
        warning: {
            "class": "lobibox-notify-warning",
            title: "Warning",
            icon: true,
            sound: true,
        },
        info: {
            "class": "lobibox-notify-info",
            title: "Information",
            icon: true,
            sound: true,
        },
        icons: {
            fontAwesome: {
                success: "fa fa-check-circle",
                error: "fa fa-times-circle",
                warning: "fa fa-exclamation-circle",
                info: "fa fa-info-circle",
            },
        },
    };
})();

