<?php
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;

class Helper extends Database {

    public function __construct()
    {

        parent::__construct();

        if (!in_array($_SERVER['HTTP_HOST'], $this->domain())) {
            //header('HTTP/1.1 403 Forbidden');
           // $this->displaySystemConfigurationMissingScreen();
           // exit();
        }

        require 'vendor/autoload.php';

    }

    public function sendEmail($to, $subject, $body): bool
    {
        global $info, $mailerHost, $mailerUsername, $mailerPassword, $mailerPort;
        $mail = new PHPMailer(true);

        try {
            //Server settings
            $mail->isSMTP();
            $mail->Host       = $mailerHost;
            $mail->SMTPAuth   = true;
            $mail->Username   = $mailerUsername;
            $mail->Password   = $mailerPassword;
            $mail->SMTPSecure = PHPMailer::ENCRYPTION_STARTTLS;
            $mail->Port       = $mailerPort;

            //Recipients
            $mail->setFrom($info['email'] ?? $mailerUsername, $info['name']);
            $mail->addAddress($to);

            $mail->addReplyTo($info['email']);

            //Content
            $mail->isHTML(true);
            $mail->Subject = $subject;
            $mail->Body    = $body;

            return $mail->send();

        } catch (Exception $e) {
            return false;
        }
    }

    public function createAccount($firstName, $lastName, $email, $password, $accountNumber,$accountType, $inputCountry, $account_currency): array
    {
        global $defaultWallet, $info;
        try {
            // Step 1: Check if the email already exists in the user_emails table
            $emailCheckQuery = "SELECT COUNT(*) FROM ".TB_EMAILS." WHERE email = :email";
            $emailCheckStmt = $this->prepare($emailCheckQuery);
            $emailCheckStmt->bindParam(':email', $email, PDO::PARAM_STR);
            $emailCheckStmt->execute();

            // Fetch the count of records with the provided email
            $emailExists = $emailCheckStmt->fetchColumn();

            if ($emailExists > 0) {
                // If the email already exists, return a message or handle accordingly
                return array("status" => false, "message" => "Email already exists!");
            } else {
                // Begin transaction for consistency across user, email, and wallet creation
                $this->beginTransaction();

                // Step 2: Insert the new account if the email doesn't exist
                $insertUserQuery = "INSERT INTO ".TB_USER." (tbl_first_name, tbl_last_name, tbl_password, tbl_country, tbl_imf, tbl_cot) VALUES (:firstName, :lastName, :password, :inputCountry, :imf, :cot)";
                $stmt = $this->prepare($insertUserQuery);

                // Bind parameters
                $stmt->bindParam(':firstName', $firstName, PDO::PARAM_STR);
                $stmt->bindParam(':lastName', $lastName, PDO::PARAM_STR);
                $stmt->bindParam(':inputCountry', $inputCountry, PDO::PARAM_STR);

                $imf = $this->generateNumber(4, 'number');
                $stmt->bindParam(':imf', $imf, PDO::PARAM_STR);

                $cot = $this->generateNumber(4, 'number');
                $stmt->bindParam(':cot', $cot, PDO::PARAM_STR);

                // Use password hashing for security
                $hashedPassword = password_hash($password, PASSWORD_BCRYPT);
                $stmt->bindParam(':password', $hashedPassword, PDO::PARAM_STR);

                // Execute the insert query for the user table
                if ($stmt->execute()) {
                    // Step 3: Get the last inserted user ID
                    $userId = $this->lastInsertId();

                    // Step 4: Insert the email into the user_emails table as the primary email
                    $insertEmailQuery = "INSERT INTO ".TB_EMAILS." (user_id, email, primary_email)
                                 VALUES (:userId, :email, :primary)";
                    $emailStmt = $this->prepare($insertEmailQuery);
                    $primaryEmail = 1; // Since this is the primary email

                    // Bind the parameters
                    $emailStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
                    $emailStmt->bindParam(':email', $email, PDO::PARAM_STR);
                    $emailStmt->bindParam(':primary', $primaryEmail, PDO::PARAM_INT);

                    // Execute the insert query for the email table
                    if ($emailStmt->execute()) {
                        // Step 5: Insert the default wallet for the user
                        $insertWalletQuery = "INSERT INTO ".TB_USER_WALLET." (user_id, balance, wallet, primary_wallet, wallet_number, account_type)
                                       VALUES (:userId, :balance, :wallet, :primaryWallet, :accountNumber, :accountType)";
                        $walletStmt = $this->prepare($insertWalletQuery);

                        // Set default wallet properties (balance: 0, wallet: 'usd', primary: 1)
                        $initialBalance = 0;
                        //$defaultWallet = 'usd';  // Assuming USD as default currency
                        $primaryWallet = 1; // Set as the primary wallet

                        // Bind wallet parameters
                        $walletStmt->bindParam(':accountType', $accountType, PDO::PARAM_STR);
                        $walletStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
                        $walletStmt->bindParam(':balance', $initialBalance, PDO::PARAM_STR);
                        $walletStmt->bindParam(':wallet', $account_currency, PDO::PARAM_STR);
                        $walletStmt->bindParam(':accountNumber', $accountNumber, PDO::PARAM_STR);
                        $walletStmt->bindParam(':primaryWallet', $primaryWallet, PDO::PARAM_INT);

                        // Execute the insert query for the wallet table
                        if ($walletStmt->execute()) {
                            // Commit the transaction if everything is successful
                            $this->commit();

                            // Step 6: Send welcome email
                            $subject = "Welcome to ".$info['name'];
                            $body = "
                            <html>
                            <head>
                                <title>Welcome to {$info['name']}</title>
                            </head>
                            <body>
                                <h2>Dear {$firstName} {$lastName},</h2>
                                <p>Thank you for creating an account with us. Below are your details:</p>
                                <ul>
                                    <li><strong>Account Number:</strong> {$accountNumber}</li>
                                    <li><strong>Account Type:</strong> {$accountType}</li>
                                    <li><strong>Account Currency:</strong> <span style='text-transform: uppercase'>{$account_currency}</span></li>
                                </ul>
                                <p>If you have any questions, feel free to contact us. Welcome aboard!</p>
                                <p>Best Regards,<br>{$info['name']}</p>
                            </body>
                            </html>
                        ";
                            $this->sendEmail($email, $subject, $body);


                            return array("status" => true, "message" => "Account created successfully!");
                        } else {
                            // Rollback transaction if wallet creation fails
                            $this->rollBack();
                            return array("status" => false, "message" => "Error creating wallet record!");
                        }
                    } else {
                        // Rollback transaction if email creation fails
                        $this->rollBack();
                        return array("status" => false, "message" => "Error creating email record!");
                    }
                } else {
                    // Rollback transaction if user creation fails
                    $this->rollBack();
                    return array("status" => false, "message" => "Error creating account!");
                }
            }
        } catch (PDOException $e) {
            // Rollback transaction on exception
            $this->rollBack();
            return array("status" => false, "message" => "Database error: " . $e->getMessage());
        }
    }

    public function authLogin_old($email, $password): array
    {
        try {
            // Step 1: Retrieve the user ID associated with the provided email
            $emailQuery = "SELECT user_id FROM ".TB_EMAILS." WHERE email = :email";
            $emailStmt = $this->prepare($emailQuery);
            $emailStmt->bindParam(':email', $email, PDO::PARAM_STR);
            $emailStmt->execute();

            // Fetch the user ID
            $userId = $emailStmt->fetchColumn();

            if (!$userId) {
                // If no user found with the provided email, return an error message
                return array("status" => false, "message" => "Email or password is not correct!");
            }

            // Step 2: Retrieve the user's hashed password using the user ID
            $passwordQuery = "SELECT tbl_password FROM ".TB_USER." WHERE tbl_id = :userId";
            $passwordStmt = $this->prepare($passwordQuery);
            $passwordStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
            $passwordStmt->execute();

            // Fetch the hashed password
            $hashedPassword = $passwordStmt->fetchColumn();

            if ($hashedPassword && password_verify($password, $hashedPassword)) {
                // If the password is correct, return a success message
                return array("status" => true, "message" => "Login successful!", "userId" => $userId);
            } else {
                // If the password is incorrect, return an error message
                return array("status" => false, "message" => "Invalid login!");
            }
        } catch (PDOException $e) {
            return array("status" => false, "message" => "Database error: " . $e->getMessage());
        }
    }

    public function authLogin($email, $password): array
    {
        try {
            // Step 1: Retrieve the user ID associated with the provided email
            $emailQuery = "SELECT user_id FROM ".TB_EMAILS." WHERE email = :email";
            $emailStmt = $this->prepare($emailQuery);
            $emailStmt->bindParam(':email', $email, PDO::PARAM_STR);
            $emailStmt->execute();

            // Fetch the user ID
            $userId = $emailStmt->fetchColumn();

            if (!$userId) {
                // If no user found with the provided email, return an error message
                return array("status" => false, "message" => "Email or password is not correct!");
            }

            // Step 2: Retrieve the user's hashed password and account status using the user ID
            $userQuery = "SELECT tbl_password, tbl_account_status FROM ".TB_USER." WHERE tbl_id = :userId";
            $userStmt = $this->prepare($userQuery);
            $userStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
            $userStmt->execute();

            // Fetch the hashed password and account status
            $user = $userStmt->fetch(PDO::FETCH_ASSOC);

            if (!$user) {
                // If no user record found, return an error message
                return array("status" => false, "message" => "Invalid login!");
            }

            // Step 3: Check the account status
            $accountStatus = $user['tbl_account_status'];
            if ($accountStatus !== 'Active') {
                if ($accountStatus === 'Under-Review') {
                    return array("status" => false, "message" => "Your account is currently under review. You will receive an email notification once the review is complete.");
                } elseif ($accountStatus === 'Archived') {
                    return array("status" => false, "message" => "Account doesn’t exist.");
                } else {
                    return array("status" => false, "message" => "Account is " . $accountStatus . ". Please contact support.");
                }
            }

            // Step 4: Verify the password
            $hashedPassword = $user['tbl_password'];
            if ($hashedPassword && password_verify($password, $hashedPassword)) {
                // If the password is correct, return a success message
                return array("status" => true, "message" => "Login successful!", "userId" => $userId);
            } else {
                // If the password is incorrect, return an error message
                return array("status" => false, "message" => "Invalid login!");
            }
        } catch (PDOException $e) {
            return array("status" => false, "message" => "Database error: " . $e->getMessage());
        }
    }

    public function getProfile($userId): array
    {
        try {
            // Step 1: Fetch user information from TB_USER
            $userQuery = "SELECT tbl_first_name as firstName, tbl_last_name as lastName, tbl_avatar as avatar, tbl_dob as dob, tbl_address as address, tbl_city as city, tbl_state as state, tbl_zipcode as zipcode, tbl_country as country, tbl_language as user_language, tbl_time_zone as time_zone, tbl_account_status as account_status, tbl_account_type as account_type, tbl_account_number as account_number, tbl_transaction_pin as transaction_pin, tbl_cot as cot, tbl_imf as imf, tbl_kyc as kyc, tbl_Last_password_changed as last_password_changed FROM ".TB_USER." WHERE tbl_id = :userId";
            $userStmt = $this->prepare($userQuery);
            $userStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
            $userStmt->execute();

            $userProfile = $userStmt->fetch(PDO::FETCH_ASSOC);

            if (!$userProfile) {
                return array("status" => false, "message" => "User not found!");
            }

            // Step 2: Fetch emails associated with the user from TB_EMAILS
            $emailsQuery = "SELECT email, primary_email FROM ".TB_EMAILS." WHERE user_id = :userId";
            $emailsStmt = $this->prepare($emailsQuery);
            $emailsStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
            $emailsStmt->execute();

            $userEmails = $emailsStmt->fetchAll(PDO::FETCH_ASSOC);


            $phoneQuery = "SELECT phone, primary_phone FROM user_phones WHERE user_id = :userId";
            $phoneStmt = $this->prepare($phoneQuery);
            $phoneStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
            $phoneStmt->execute();

            $userPhones = $phoneStmt->fetchAll(PDO::FETCH_ASSOC);


            $docQuery = "SELECT doc_id as id, doc_file as file, doc_status as status, created_at, doc_size, doc_type as type FROM user_documents WHERE user_id = :userId";
            $docStmt = $this->prepare($docQuery);
            $docStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
            $docStmt->execute();

            $userDoc = $docStmt->fetchAll(PDO::FETCH_ASSOC);


            // Step 3: Fetch wallet information associated with the user from TB_USER_WALLET
            $walletQuery = "SELECT wallet_id, balance, wallet, primary_wallet, wallet_number, account_type,account_status, loan_balance FROM ".TB_USER_WALLET." WHERE user_id = :userId";
            $walletStmt = $this->prepare($walletQuery);
            $walletStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
            $walletStmt->execute();
            $userWallets = $walletStmt->fetchAll(PDO::FETCH_ASSOC);


            $cardQuery = "SELECT vc_id as id, vc_numer as number, vc_name as name, vc_valid as valid, vc_type as type, vc_created_at as created_at, vc_status as card_status, vc_card_option as card_option, wallet_number as account_number, vc_cvv as cvv FROM virtual_cards c LEFT JOIN wallet w on w.wallet_id=c.wallet_id WHERE c.user_id = :userId";
            $cardStmt = $this->prepare($cardQuery);
            $cardStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
            $cardStmt->execute();
            $userCards = $cardStmt->fetchAll(PDO::FETCH_ASSOC);



            // Step 4: Prepare the full profile response
            $userProfile['emails'] = $userEmails; // Attach emails to user profile
            $userProfile['wallets'] = $userWallets; // Attach wallets to user profile
            $userProfile['phones'] = $userPhones; // Attach wallets to user profile
            $userProfile['cards'] = $userCards; // Attach wallets to user profile
            $userProfile['documents'] = $userDoc; // Attach doc to user profile

            return array("status" => true, "profile" => $userProfile);

        } catch (PDOException $e) {
            return array("status" => false, "message" => "Database error: " . $e->getMessage());
        }
    }

    function formatFileSize($size) {
        if ($size < 1024) {
            return $size . ' bytes';
        } elseif ($size < 1048576) { // 1024 * 1024
            return round($size / 1024, 2) . ' KB';
        } elseif ($size < 1073741824) { // 1024 * 1024 * 1024
            return round($size / 1048576, 2) . ' MB';
        } else {
            return round($size / 1073741824, 2) . ' GB';
        }
    }

    function formatDate($date) {
        // Convert the date string into a timestamp
        $timestamp = strtotime($date);

        // Format the timestamp into the desired format
        return date('D d M, Y', $timestamp);
    }

    function formatDocumentType($value) {
        // Replace hyphens with spaces
        $formattedValue = str_replace('-', ' ', $value);

        // Capitalize each word
        return ucwords($formattedValue);
    }

    function domain(){
        return ['localhost'];
    }

    function docStatus($status){
        switch ($status){
            case "Pending":
                $x = '<span class="text-nowrap text-info" style="font-size: 13px">Under Review</span>';
                break;

            case "Approved":
                $x =  '<span class="text-nowrap text-success" style="font-size: 13px">Approved</span>';
                break;

            case "Rejected":
                $x =  '<span class="text-nowrap text-danger" style="font-size: 13px">Rejected</span>';
                break;

            default:
                $x = '<span class="text-nowrap text-info" style="font-size: 13px">Under Review</span>';
        }
        return $x;
    }

    public function insertDocument($userId, $file, $size, $type){
        $stmt = $this->prepare("INSERT INTO user_documents (doc_file, user_id, doc_size, doc_type) VALUES (:file, :id, :size, :type)");
        $stmt->bindParam(":id", $userId);
        $stmt->bindParam(":file", $file);
        $stmt->bindParam(":size", $size);
        $stmt->bindParam(":type", $type);
        $stmt->execute();

        if($stmt->rowCount() > 0){
            return array(
                "status" => true,
                "message" => "File uploaded successfully"
            );
        } else {
            return array(
                "status" => false,
                "message" => "Oops, something went wrong, try again"
            );
        }
    }


    function usersProfile(){
        $userQuery = "SELECT tbl_id as _id, tbl_first_name as firstName, tbl_last_name as lastName, tbl_avatar as avatar, tbl_dob as dob, tbl_address as address, tbl_city as city, tbl_state as state, tbl_zipcode as zipcode, tbl_country as country, tbl_language as user_language, tbl_time_zone as time_zone, tbl_account_status as account_status, tbl_account_type as account_type, tbl_account_number as account_number, tbl_transaction_pin as transaction_pin, tbl_cot as cot, tbl_imf as imf, tbl_kyc as kyc, tbl_Last_password_changed as last_password_changed FROM ".TB_USER." ";
        $userStmt = $this->prepare($userQuery);
        $userStmt->execute();

        $array = [];
        while ($row = $userStmt->fetch(PDO::FETCH_ASSOC)){
            $array[] = $row;
        }
        return $array;
    }

    function allUsers(){
        $stmt = $this->prepare("SELECT tbl_first_name as first_name, tbl_last_name as last_name, wallet_number, balance, wallet as currency, user_id FROM ".TB_USER." u RIGHT JOIN wallet w on tbl_id=user_id WHERE tbl_id=w.user_id");
        $stmt->execute();
        $array = [];
        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
            $array[] = $row;
        }
        return $array;
    }

    public function changePassword($userId, $existingPassword, $newPassword){
        global $info;
        // Step 1: Retrieve the user's current password from the database
        $stmt = $this->prepare("SELECT tbl_password, email, tbl_first_name FROM ".TB_USER." LEFT JOIN user_emails on user_id=tbl_id WHERE tbl_id = :user_id AND primary_email =1");
        $stmt->bindParam(":user_id", $userId);
        $stmt->execute();

        // Fetch user data
        $userData = $stmt->fetch(PDO::FETCH_ASSOC);

        if ($userData) {
            // Step 2: Verify the existing password
            if (password_verify($existingPassword, $userData['tbl_password'])) {

                // Step 3: Hash the new password
                $hashedNewPassword = password_hash($newPassword, PASSWORD_DEFAULT);

                // Step 4: Update the password in the database
                $updateStmt = $this->prepare("UPDATE ".TB_USER." SET tbl_password = :new_password, tbl_Last_password_changed = :last WHERE tbl_id = :user_id");
                $updateStmt->bindParam(":new_password", $hashedNewPassword);
                $date = date('d M Y h:i:s a');
                $updateStmt->bindParam(":last", $date);
                $updateStmt->bindParam(":user_id", $userId);
                if ($updateStmt->execute()) {

                    // Step 5: Send an email notification to the user about the password change
                    $to = $userData['email'];
                    $subject = "Password Change Notification";
                    $message = "Dear ".$userData['tbl_first_name'].",\n\nYour password has been successfully changed.\n\nIf you did not make this change, please contact support immediately.\n\nBest Regards,\nSupport Team";
                    $headers = "From: ".$info['email'];
                    mail($to, $subject, $message, $headers);

                    // Step 6: Return a success response
                    return array(
                        "status" => true,
                        "message" => "Password changed successfully."
                    );
                } else {
                    return array(
                        "status" => false,
                        "message" => "Failed to update password."
                    );
                }
            } else {
                // Password doesn't match
                return array(
                    "status" => false,
                    "message" => "Incorrect existing password."
                );
            }
        } else {
            // User not found
            return array(
                "status" => false,
                "message" => "User not found."
            );
        }
    }

    function profileUpdate($firstName, $lastName, $dob, $address, $inputCountry, $state, $city, $zipCode, $user_id){
        $stmt = $this->prepare("UPDATE ".TB_USER." SET tbl_first_name = :firstName, tbl_last_name =:lastName, tbl_dob =:dob, tbl_address =:address, tbl_country=:country, tbl_state=:state, tbl_city=:city, tbl_zipcode=:zip WHERE tbl_id = :user_id");
        $stmt->bindParam(":user_id", $user_id);
        $stmt->bindParam(":firstName", $firstName);
        $stmt->bindParam(":lastName", $lastName);
        $stmt->bindParam(":dob", $dob);
        $stmt->bindParam(":address", $address);
        $stmt->bindParam(":country", $inputCountry);
        $stmt->bindParam(":state", $state);
        $stmt->bindParam(":city", $city);
        $stmt->bindParam(":zip", $zipCode);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "status"=>true,
                "message"=>"Profile updated successfully"
            );
        }else{
            return array(
                "status"=>false,
                "message"=>"Oops, something went wrong while trying to update profile"
            );
        }
    }

    function generateExpirationDate($yearsToAdd = 5) {
        // Get the current date
        $currentDate = new DateTime();

        // Add the specified number of years (default is 5)
        $currentDate->modify("+{$yearsToAdd} years");

        // Format the date to 'mm/yy'
        return $currentDate->format('m/y');
    }

    function getDialingCode($codeString) {
        // Split the string by commas
        $parts = explode(',', $codeString);

        // Check if there are at least 2 parts (country code and dialing code)
        if (count($parts) >= 2) {
            // Return the second part which is the dialing code
            return trim($parts[1]);
        } else {
            return null; // In case the format is unexpected
        }
    }

    function addPhoneNumber($userId, $phone){

        // Step 1: Check if the user already has a phone number marked as primary
        $stmt = $this->prepare("SELECT primary_phone FROM user_phones WHERE user_id = :user_id AND primary_phone = 1");
        $stmt->bindParam(':user_id', $userId);
        $stmt->execute();

        // Step 2: Set the primary_phone status for the new phone
        $primaryPhone = ($stmt->rowCount() > 0) ? 0 : 1;

        // Step 3: Insert the new phone number with appropriate primary_phone value
        $insertStmt = $this->prepare("INSERT INTO user_phones (user_id, phone, primary_phone) VALUES (:user_id, :phone, :primary_phone)");
        $insertStmt->bindParam(':user_id', $userId);
        $insertStmt->bindParam(':phone', $phone);
        $insertStmt->bindParam(':primary_phone', $primaryPhone);

        if($insertStmt->execute()){
            return array("status" => true, "message" => "Phone number added successfully.");
        } else {
            return array("status" => false, "message" => "Failed to add phone number.");
        }
    }

    public function createCard($cardNumber, $name, $type, $card_method, $wallet_id, $userId, $cvv){

        if (!in_array($card_method, ['Credit', 'Debit'])) {
            return array(
                "status" => false,
                "message" => "Invalid card method. Must be either 'Credit' or 'Debit'."
            );
        }

        // SQL query to insert the card details into the database
        $cardQuery = "INSERT INTO virtual_cards (vc_numer, vc_name, vc_valid, vc_type, vc_card_option, wallet_id, user_id, vc_cvv) 
                  VALUES (:cardNumber, :name, :valid, :type, :card_method, :wallet_id, :userId, :cvv)";

        // Prepare the SQL statement
        $cardStmt = $this->prepare($cardQuery);

        // Bind the parameters
        $cardStmt->bindParam('cardNumber', $cardNumber, PDO::PARAM_STR);
        $cardStmt->bindParam(':name', $name, PDO::PARAM_STR);
        $cardStmt->bindParam(':type', $type, PDO::PARAM_STR);
        $cardStmt->bindParam(':card_method', $card_method, PDO::PARAM_STR);
        $cardStmt->bindParam(':wallet_id', $wallet_id, PDO::PARAM_INT);
        $cardStmt->bindParam(':userId', $userId, PDO::PARAM_INT);
        $cardStmt->bindParam(':cvv', $cvv, PDO::PARAM_STR);

        // Generate the card's expiration date (default 5 years from now)
        $valid = $this->generateExpirationDate();

        // Bind the expiration date
        $cardStmt->bindParam(':valid', $valid, PDO::PARAM_STR); // Use PARAM_STR since the date is in 'mm/yy' format

        // Execute the statement
        $cardStmt->execute();

        if($cardStmt->rowCount()>0){
            return array(
                "status"=>true,
                "message"=>"Card created successfully"
            );
        }else{
            return array(
                "status"=>false,
                "message"=>"Oops, something went wrong"
            );
        }
    }

    public function getPrimaryWallet(array $wallets): ?array
    {
        foreach ($wallets as $wallet) {
            if ($wallet['primary_wallet'] == 1) {
                return $wallet; // Return the wallet where primary_wallet = 1
            }
        }

        // If no primary wallet is found, return null or handle accordingly
        return null;
    }

    public function getWalletSymbol(string $walletType): string
    {
        // Define the mapping between wallet type and its symbol
        $walletSymbols = [
            'usd' => '$',
            'euro' => '€',
            'pound' => '£'
        ];

        // Return the corresponding symbol or a default symbol if wallet type is not found
        return $walletSymbols[$walletType] ?? '?'; // Return '?' if wallet type is not recognized
    }

    function parseTransactionData($data) {
        // Split the string by "-"
        $parts = explode('-', $data);

        // Ensure we have exactly 3 parts
        if (count($parts) !== 4) {
            return ['error' => 'Invalid data format'];
        }

        // Map the parts to an associative array
        return [
            'name' => $parts[0],          // Name
            'account_number' => $parts[1], // Account number
            'currency' => $parts[2],      // Currency code
            'user_id' => $parts[3]        // Currency code
        ];
    }

    function formatAccountStatus($status)
    {
        $statusMap = [
            'Active' => '<p class="col-sm-9 text-3"><span class="bg-success text-white rounded-pill d-inline-block px-2 mb-0"><i class="fas fa-check-circle"></i> Active</span></p>',
            'Dormant' => '<p class="col-sm-9 text-3"><span class="bg-warning text-dark rounded-pill d-inline-block px-2 mb-0"><i class="fas fa-clock"></i> Dormant</span></p>',
            'Frozen' => '<p class="col-sm-9 text-3"><span class="bg-info text-white rounded-pill d-inline-block px-2 mb-0"><i class="fas fa-snowflake"></i> Frozen</span></p>',
            'Locked' => '<p class="col-sm-9 text-3"><span class="bg-secondary text-white rounded-pill d-inline-block px-2 mb-0"><i class="fas fa-lock"></i> Locked</span></p>',
            'Banned' => '<p class="col-sm-9 text-3"><span class="bg-danger text-white rounded-pill d-inline-block px-2 mb-0"><i class="fas fa-ban"></i> Banned</span></p>',
            'Under-Review' => '<p class="col-sm-9 text-3"><span class="bg-primary text-white rounded-pill d-inline-block px-2 mb-0"><i class="fas fa-search"></i> Under Review</span></p>',
            'Archived' => '<p class="col-sm-9 text-3"><span class="bg-dark text-white rounded-pill d-inline-block px-2 mb-0"><i class="fas fa-archive"></i> Archived</span></p>',
        ];

        // Default to a generic status if not recognized
        return $statusMap[$status] ?? '<span class="bg-light text-dark rounded-pill d-inline-block px-2 mb-0"><i class="fas fa-question-circle"></i> Unknown Status</span>';
    }

    public function verifyAccountNumber($accountNumber, $currencyType) {
        // Prepare the SQL statement
        $stmt = $this->prepare("SELECT wallet_number, user_id, wallet, tbl_first_name AS firstName, tbl_last_name AS lastName 
                            FROM " . TB_USER_WALLET . " 
                            LEFT JOIN " . TB_USER . " ON user_id = tbl_id 
                            WHERE wallet_number = :wallet_number AND wallet = :type");

        // Bind parameters
        $stmt->bindParam(":wallet_number", $accountNumber);
        $stmt->bindParam(":type", $currencyType);

        // Execute the statement
        $stmt->execute();

        // Fetch the result
        $result = $stmt->fetch(PDO::FETCH_ASSOC);

        // Check if a result was found and prepare the response
        if ($result) {
            return [
                'status' => true,
                'message' => 'Account verified successfully.',
                'data' => $result // Optionally include the data
            ];
        } else {
            return [
                'status' => false,
                'message' => 'Account not found.'
            ];
        }
    }

    public function handleWireTransfer($accountName, $amount, $bankName, $purposeOfTransfer, $routingNo, $selectedCountry, $swiftCode, $transferCode, $currency, $userId, $accountNumber) {
        global $info;
        // Verify user with transaction PIN
        $stmt = $this->prepare("SELECT * FROM ".TB_USER." WHERE tbl_transaction_pin = :pin AND tbl_id = :user_id");
        $stmt->bindParam(":pin", $transferCode);
        $stmt->bindParam(":user_id", $userId);
        $stmt->execute();

        if($stmt->rowCount() > 0) {
            // Prepare transaction details
            $description = $purposeOfTransfer . "/" . $selectedCountry . '/' . $swiftCode . '-' . $routingNo . ' to ' . $accountNumber . '/' . $accountName;
            $transactionId = 'TRX' . time() . rand(1000, 9999);
            $currentlyRequired = 'otp';

            // Insert transaction
            $stmtCreate = $this->prepare(
                "INSERT INTO transactions (user_id, transaction_date, transaction_type, amount, currency, description, status, bank_name, transaction_id, transferOption, currentlyRequired, created_at) 
             VALUES (:user_id, NOW(), 'withdrawal', :amount, :currency, :description, 'pending', :bankName, :transaction_id, 'Wire', :currentlyRequired, :created_at)"
            );

            // Bind parameters
            $stmtCreate->bindParam(":user_id", $userId);
            $stmtCreate->bindParam(":amount", $amount);
            $stmtCreate->bindParam(":currency", $currency);
            $stmtCreate->bindParam(":description", $description);
            $stmtCreate->bindParam(":bankName", $bankName);
            $stmtCreate->bindParam(":transaction_id", $transactionId);
            $stmtCreate->bindParam(":currentlyRequired", $currentlyRequired);
            $created_at = date('Y-m-d H:i:s');
            $stmtCreate->bindParam(":created_at", $created_at);

            // Execute the transaction insertion
            $stmtCreate->execute();

            if($stmtCreate->rowCount() > 0) {
                // Generate and update OTP for the user
                $stmtOTP = $this->prepare("UPDATE ".TB_USER." SET tbl_otp_expire = :expiry, tbl_otp = :otp WHERE tbl_id = :user_id");

                $number = $this->generateNumber(4, 'number'); // Generate a 4-digit OTP
                $expiry = date('Y-m-d H:i:s', strtotime('+1 hour')); // Set OTP expiration time to 1 hour from now

                $stmtOTP->bindParam(":user_id", $userId);
                $stmtOTP->bindParam(":otp", $number);
                $stmtOTP->bindParam(":expiry", $expiry);
                $stmtOTP->execute();

                // Fetch primary email from user_emails
                $stmtUserEmail = $this->prepare("SELECT email FROM user_emails WHERE user_id = :user_id AND primary_email = 1");
                $stmtUserEmail->bindParam(":user_id", $userId);
                $stmtUserEmail->execute();
                $userEmail = $stmtUserEmail->fetchColumn();

                // Send OTP to user's primary email
                if ($userEmail) {
                    // Prepare email details
                    $subject = "Your OTP Code";
                    $message = "Dear user,\n\nYour One-Time Password (OTP) for your wire transfer is: " . $number . ".\n\nPlease use this code to proceed with your transaction. The OTP will expire in 1 hour.\n\nThank you!";
                    $headers = "From: ".$info['email']."\r\n";
                    $headers .= "Reply-To: ".$info['email']."\r\n";
                    $headers .= "X-Mailer: PHP/" . phpversion();

                    // Use PHP's mail function to send the OTP
//                    if (mail($userEmail, $subject, $message, $headers)) {
                        if($this->sendEmail($userEmail, $subject, $message)){
                        // Email sent successfully
                        return array(
                            "status" => true,
                            "message" => "Your request has been processed, and an OTP has been sent to your email. You can proceed.",
                            "transaction_id" => $transactionId,
                            "receiver" => $accountName.'/'.$accountNumber,
                            "amount" => $currency.$amount,

                        );
                    } else {
                        // Email sending failed
                        return array(
                            "status" => false,
                            "message" => "Transaction created, but failed to send OTP to your email."
                        );
                    }
                } else {
                    // No primary email found
                    return array(
                        "status" => false,
                        "message" => "No primary email found. Please update your email settings."
                    );
                }
            } else {
                return array(
                    "status" => false,
                    "message" => "Oops, something went wrong. We could not process your request."
                );
            }
        } else {
            return array(
                "status" => false,
                "message" => "Invalid transaction PIN"
            );
        }
    }

    public function handleDomesticTransfer($accountName, $amount, $bankName, $purposeOfTransfer, $transferCode, $currency, $userId, $accountNumber){

        global $info;

        $stmt = $this->prepare("SELECT * FROM ".TB_USER." WHERE tbl_transaction_pin = :pin AND tbl_id = :user_id");
        $stmt->bindParam(":pin", $transferCode);
        $stmt->bindParam(":user_id", $userId);
        $stmt->execute();

        if($stmt->rowCount() > 0) {

            $description = $purposeOfTransfer . "/" . $bankName . '/' . $accountName . '-' . $amount . ' to ' . $accountNumber . '/' . $accountName;
            $transactionId = 'TRX' . time() . rand(1000, 9999);
            $currentlyRequired = 'otp';


            // Insert transaction
            $stmtCreate = $this->prepare(
                "INSERT INTO transactions (user_id, transaction_date, transaction_type, amount, currency, description, status, bank_name, transaction_id, transferOption, currentlyRequired,created_at) 
             VALUES (:user_id, NOW(), 'withdrawal', :amount, :currency, :description, 'pending', :bankName, :transaction_id, 'Domestic', :currentlyRequired,:created_at)"
            );

            // Bind parameters
            $stmtCreate->bindParam(":user_id", $userId);
            $stmtCreate->bindParam(":amount", $amount);
            $stmtCreate->bindParam(":currency", $currency);
            $stmtCreate->bindParam(":description", $description);
            $stmtCreate->bindParam(":bankName", $bankName);
            $stmtCreate->bindParam(":transaction_id", $transactionId);
            $stmtCreate->bindParam(":currentlyRequired", $currentlyRequired);
            $created_at = date('Y-m-d H:i:s');
            $stmtCreate->bindParam(":created_at", $created_at);

            // Execute the transaction insertion
            $stmtCreate->execute();

            if($stmtCreate->rowCount() > 0) {

                $stmtOTP2 = $this->prepare("UPDATE ".TB_USER." SET tbl_otp_expire = :expiry, tbl_otp = :otp WHERE tbl_id = :user_id");

                $number = $this->generateNumber(4, 'number'); // Generate a 4-digit OTP
                $expiry = date('Y-m-d H:i:s', strtotime('+1 hour')); // Set OTP expiration time to 1 hour from now

                $stmtOTP2->bindParam(":user_id", $userId);
                $stmtOTP2->bindParam(":otp", $number);
                $stmtOTP2->bindParam(":expiry", $expiry);
                $stmtOTP2->execute();

                // Fetch primary email from user_emails
                $stmtUserEmail = $this->prepare("SELECT email FROM user_emails WHERE user_id = :user_id AND primary_email = 1");
                $stmtUserEmail->bindParam(":user_id", $userId);
                $stmtUserEmail->execute();
                $userEmail = $stmtUserEmail->fetchColumn();

                // Send OTP to user's primary email
                if ($userEmail) {
                    // Prepare email details
                    $subject = "Your OTP Code";
                    $message = "Dear user,\n\nYour One-Time Password (OTP) for your wire transfer is: " . $number . ".\n\nPlease use this code to proceed with your transaction. The OTP will expire in 1 hour.\n\nThank you!";
                    $headers = "From: ".$info['email']."\r\n";
                    $headers .= "Reply-To: ".$info['email']."\r\n";
                    $headers .= "X-Mailer: PHP/" . phpversion();

                    // Use PHP's mail function to send the OTP
                   // if (mail($userEmail, $subject, $message, $headers)) {
                    if ($this->sendEmail($userEmail, $subject, $message)) {
                        // Email sent successfully
                        return array(
                            "status" => true,
                            "message" => "Your request has been processed, and an OTP has been sent to your email. You can proceed.",
                            "transaction_id" => $transactionId,
                            "receiver" => $accountName.'/'.$accountNumber,
                            "amount" => $currency.$amount,

                        );
                    } else {
                        // Email sending failed
                        return array(
                            "status" => false,
                            "message" => "Transaction created, but failed to send OTP to your email."
                        );
                    }
                } else {
                    // No primary email found
                    return array(
                        "status" => false,
                        "message" => "No primary email found. Please update your email settings."
                    );
                }


            }else{

                return array(
                    "status" => false,
                    "message" => "Oops, something went wrong. We could not process your request."
                );
            }



        }else{

            return array(
                "status" => false,
                "message" => "Invalid transaction PIN"
            );

        }

    }


    function adminWireTransfer($user_id,$currency, $senderAccountNumber, $senderName, $domesticBankName, $beneficiaryAccountNumber, $beneficiaryAccountName, $amount, $purposeOfTransfer, $created_at, $transferStatus, $transferRequired,$domesticTransactionType, $wireRoutingNo, $wireSwiftCode, $wireCountry){
        $stmt = $this->prepare("INSERT INTO transactions (user_id, transaction_type, amount,currency,description,status,bank_name,transaction_id,transferOption,currentlyRequired,created_at) VALUES (:user_id,:withdrawal, :amount, :currency,:description,:status,:bank_name,:transaction_id,'Wire',:currentlyRequired,:created_at)");
        $stmt->bindParam(":user_id", $user_id);
        $stmt->bindParam(":amount", $amount);
        $stmt->bindParam(":withdrawal", $domesticTransactionType);
        $stmt->bindParam(":currency", $currency);

        if($domesticTransactionType ==='credit'){
            $description = $purposeOfTransfer.'/'.$wireCountry.'/'.$wireRoutingNo.'-'.$wireSwiftCode.' from '.$beneficiaryAccountNumber.'/'.$beneficiaryAccountName;
        }else{
            $description = $purposeOfTransfer.'/'.$wireCountry.'/'.$wireRoutingNo.'-'.$wireSwiftCode.' from '.$beneficiaryAccountNumber.'/'.$beneficiaryAccountName;
        }

        $stmt->bindParam(":description", $description);
        $stmt->bindParam(":status", $transferStatus);
        $stmt->bindParam(":bank_name", $domesticBankName);
        $transactionId = 'TRX' . time() . rand(1000, 9999);
        $stmt->bindParam(":transaction_id", $transactionId);
        $stmt->bindParam(":currentlyRequired", $transferRequired);

        $stmt->bindParam(":created_at", $created_at);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "success"=>true,
                "message"=>"Transfer processed successfully"
            );
        }else{
            return array(
                "success"=>false,
                "message"=>"Oops, something went wrong, try again"
            );
        }
    }

    function deleteUserAccount_old($id){
        $stmt = $this->prepare("DELETE FROM ".TB_USER." WHERE tbl_id = :id");
        $stmt->bindParam(":id", $id);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "success"=>true,
                "message"=>"Account deleted successfully"
            );
        }else{
            return array(
                "success"=>false,
                "message"=>"Oops, something went wrong, try again"
            );
        }
    }

    function statistics() {
        $stats = [];

        // Count the number of records in the users table
        $stmt = $this->prepare("SELECT COUNT(tbl_id) AS total_users FROM users");
        $stmt->execute();
        $stats['total_users'] = $stmt->fetch(PDO::FETCH_ASSOC)['total_users'];

        // Count the number of active users in the users table
        $stmt = $this->prepare("SELECT COUNT(tbl_id) AS active_users FROM users WHERE tbl_account_status = 'Active'");
        $stmt->execute();
        $stats['active_users'] = $stmt->fetch(PDO::FETCH_ASSOC)['active_users'];

        // Count the number of active virtual cards in the virtual_cards table
        $stmt = $this->prepare("SELECT COUNT(vc_id) AS active_virtual_cards FROM virtual_cards WHERE vc_status = 'Active'");
        $stmt->execute();
        $stats['active_virtual_cards'] = $stmt->fetch(PDO::FETCH_ASSOC)['active_virtual_cards'];

        // Count the number of pending transactions in the transactions table
        $stmt = $this->prepare("SELECT COUNT(id) AS pending_transactions FROM transactions WHERE status = 'pending'");
        $stmt->execute();
        $stats['pending_transactions'] = $stmt->fetch(PDO::FETCH_ASSOC)['pending_transactions'];

        // Return the counts as an array
        return $stats;
    }


    function deleteUserAccount($id) {
        $imagesToDelete = [];

        // Define base paths for images
        $avatarPath = '../../../profile/images/avatars/'; // Path where avatar images are stored
        $documentPath = '../../../profile/images/documents/'; // Path where document images are stored

        // Check if user has an avatar in the TB_USER table
        $stmt = $this->prepare("SELECT tbl_avatar FROM " . TB_USER . " WHERE tbl_id = :id");
        $stmt->bindParam(":id", $id);
        $stmt->execute();
        $user = $stmt->fetch(PDO::FETCH_ASSOC);

        // If tbl_avatar is not null, add the avatar full path to $imagesToDelete
        if ($user && !empty($user['tbl_avatar'])) {
            $imagesToDelete[] = $avatarPath . $user['tbl_avatar'];
        }

        // Check for user documents in user_documents table
        $stmt = $this->prepare("SELECT doc_file FROM user_documents WHERE user_id = :id");
        $stmt->bindParam(":id", $id);
        $stmt->execute();
        $documents = $stmt->fetchAll(PDO::FETCH_ASSOC);

        // If documents exist, add their full paths to $imagesToDelete
        foreach ($documents as $document) {
            $imagesToDelete[] = $documentPath . $document['doc_file'];
        }

        // Delete the user record from TB_USER
        $stmt = $this->prepare("DELETE FROM " . TB_USER . " WHERE tbl_id = :id");
        $stmt->bindParam(":id", $id);
        $stmt->execute();

        // If the user record was successfully deleted
        if ($stmt->rowCount() > 0) {
//            // Delete user documents related to the user
//            $stmt = $this->prepare("DELETE FROM user_documents WHERE user_id = :id");
//            $stmt->bindParam(":id", $id);
//            $stmt->execute();

            // Unlink (delete) each image file with full path
            foreach ($imagesToDelete as $filePath) {
                if (file_exists($filePath)) {
                    unlink($filePath);
                }
            }

            return array(
                "success" => true,
                "message" => "Account and associated images deleted successfully"
            );
        } else {
            return array(
                "success" => false,
                "message" => "Oops, something went wrong, try again"
            );
        }
    }


    function fundAccount($amount, $wallet){
        $stmt = $this->prepare("UPDATE wallet SET balance = (balance+:amount) WHERE wallet_number = :wallet");
        $stmt->bindParam(":amount", $amount);
        $stmt->bindParam(":wallet", $wallet);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "success" => true,
                "message" => "Account has been funded successfully"
            );
        }else{
            return array(
                "success" => false,
                "message" => "Oops, something went wrong, try again"
            );
        }
    }

    function changeWalletStatus($wallet_status, $wallet){
        $stmt = $this->prepare("UPDATE wallet SET account_status = :status WHERE wallet_number = :wallet");
        $stmt->bindParam(":status", $wallet_status);
        $stmt->bindParam(":wallet", $wallet);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "success" => true,
                "message" => "Wallet status has been changed successfully"
            );
        }else{
            return array(
                "success" => false,
                "message" => "Oops, something went wrong, try again"
            );
        }
    }


    function changeUserAccountStatus($status, $id){
        $stmt = $this->prepare("UPDATE ".TB_USER." SET tbl_account_status = :status WHERE tbl_id = :id");
        $stmt->bindParam(":status", $status);
        $stmt->bindParam(":id", $id);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "success" => true,
                "message" => "User account status has been changed successfully"
            );
        }else{
            return array(
                "success" => false,
                "message" => "Oops, something went wrong, try again"
            );
        }

    }

    function changeCardStatus($status, $id){
        $stmt = $this->prepare("UPDATE virtual_cards SET vc_status = :status WHERE vc_id = :id");
        $stmt->bindParam(":status", $status);
        $stmt->bindParam(":id", $id);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "success" => true,
                "message" => "Card status has been changed successfully"
            );
        }else{
            return array(
                "success" => false,
                "message" => "Oops, something went wrong, try again"
            );
        }
    }

    function changeTransactionStatus($status, $transactionNo){
        $stmt = $this->prepare("UPDATE transactions SET status = :status WHERE transaction_id = :transaction_id");
        $stmt->bindParam(":status", $status);
        $stmt->bindParam(":transaction_id", $transactionNo);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "success" => true,
                "message" => "Transaction status has been changed successfully"
            );
        }else{
            return array(
                "success" => false,
                "message" => "Oops, something went wrong, try again"
            );
        }
    }


    function changeDocumentStatus($status, $id){
        $stmt = $this->prepare("UPDATE user_documents SET doc_status = :status WHERE user_id = :id");
        $stmt->bindParam(":status", $status);
        $stmt->bindParam(":id", $id);
        $stmt->execute();
        if($stmt->rowCount()>0){

            if($status =='Pending'){
                $s = "Required";
            }elseif($status =="Rejected"){
                $s = "Rejected";
            }else if($status=='Approved'){
                $s = "Approved";
            }

            $stmt = $this->prepare("UPDATE ".TB_USER." SET tbl_kyc = :status WHERE tbl_id = :id");
            $stmt->bindParam(":status", $s);
            $stmt->bindParam(":id", $id);
            $stmt->execute();

            return array(
                "success" => true,
                "message" => "Document status has been changed successfully"
            );


        }else{
            return array(
                "success" => false,
                "message" => "Oops, something went wrong, try again"
            );
        }
    }


    function adminDomesticTransfer($user_id,$currency, $senderAccountNumber, $senderName, $domesticBankName, $beneficiaryAccountNumber, $beneficiaryAccountName, $amount, $purposeOfTransfer, $created_at, $transferStatus, $transferRequired,$domesticTransactionType){
        $stmt = $this->prepare("INSERT INTO transactions (user_id, transaction_type, amount,currency,description,status,bank_name,transaction_id,transferOption,currentlyRequired,created_at) VALUES (:user_id,:withdrawal, :amount, :currency,:description,:status,:bank_name,:transaction_id,'Domestic',:currentlyRequired,:created_at)");
        $stmt->bindParam(":user_id", $user_id);
        $stmt->bindParam(":amount", $amount);
        $stmt->bindParam(":withdrawal", $domesticTransactionType);
        $stmt->bindParam(":currency", $currency);

        if($domesticTransactionType ==='credit'){
            $description = $purposeOfTransfer.'/'.$domesticBankName.'/'.$senderName.' from '.$beneficiaryAccountNumber.'/'.$beneficiaryAccountName;
        }else{
            $description = $purposeOfTransfer.'/'.$domesticBankName.'/'.$senderName.' to '.$beneficiaryAccountNumber.'/'.$beneficiaryAccountName;
        }

        $stmt->bindParam(":description", $description);
        $stmt->bindParam(":status", $transferStatus);
        $stmt->bindParam(":bank_name", $domesticBankName);
        $transactionId = 'TRX' . time() . rand(1000, 9999);
        $stmt->bindParam(":transaction_id", $transactionId);
        $stmt->bindParam(":currentlyRequired", $transferRequired);

        $stmt->bindParam(":created_at", $created_at);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "success"=>true,
                "message"=>"Transfer processed successfully"
            );
        }else{
            return array(
                "success"=>false,
                "message"=>"Oops, something went wrong, try again"
            );
        }
    }

    function countries(){
        return [
            "Aaland Islands",
            "Afghanistan",
            "Albania",
            "Algeria",
            "American Samoa",
            "Andorra",
            "Angola",
            "Anguilla",
            "Antarctica",
            "Antigua and Barbuda",
            "Argentina",
            "Armenia",
            "Aruba",
            "Ascension Island (British)",
            "Australia",
            "Austria",
            "Azerbaijan",
            "Bahamas",
            "Bahrain",
            "Bangladesh",
            "Barbados",
            "Belarus",
            "Belgium",
            "Belize",
            "Benin",
            "Bermuda",
            "Bhutan",
            "Bolivia",
            "Bonaire, Sint Eustatius and Saba",
            "Bosnia and Herzegovina",
            "Botswana",
            "Bouvet Island",
            "Brazil",
            "British Indian Ocean Territory",
            "Brunei Darussalam",
            "Bulgaria",
            "Burkina Faso",
            "Burundi",
            "Cambodia",
            "Cameroon",
            "Canada",
            "Canary Islands",
            "Cape Verde",
            "Cayman Islands",
            "Central African Republic",
            "Chad",
            "Chile",
            "China",
            "Christmas Island",
            "Cocos (Keeling) Islands",
            "Colombia",
            "Comoros",
            "Congo",
            "Cook Islands",
            "Costa Rica",
            "Cote D'Ivoire",
            "Croatia",
            "Cuba",
            "Curacao",
            "Cyprus",
            "Czech Republic",
            "Democratic Republic of Congo",
            "Denmark",
            "Djibouti",
            "Dominica",
            "Dominican Republic",
            "East Timor",
            "Ecuador",
            "Egypt",
            "El Salvador",
            "Equatorial Guinea",
            "Eritrea",
            "Estonia",
            "Ethiopia",
            "Falkland Islands (Malvinas)",
            "Faroe Islands",
            "Fiji",
            "Finland",
            "France, Metropolitan",
            "French Guiana",
            "French Polynesia",
            "French Southern Territories",
            "FYROM",
            "Gabon",
            "Gambia",
            "Georgia",
            "Germany",
            "Ghana",
            "Gibraltar",
            "Greece",
            "Greenland",
            "Grenada",
            "Guadeloupe",
            "Guam",
            "Guatemala",
            "Guernsey",
            "Guinea",
            "Guinea-Bissau",
            "Guyana",
            "Haiti",
            "Heard and Mc Donald Islands",
            "Honduras",
            "Hong Kong",
            "Hungary",
            "Iceland",
            "India",
            "Indonesia",
            "Iran (Islamic Republic of)",
            "Iraq",
            "Ireland",
            "Isle of Man",
            "Israel",
            "Italy",
            "Jamaica",
            "Japan",
            "Jersey",
            "Jordan",
            "Kazakhstan",
            "Kenya",
            "Kiribati",
            "Korea, Republic of",
            "Kosovo, Republic of",
            "Kuwait",
            "Kyrgyzstan",
            "Lao People's Democratic Republic",
            "Latvia",
            "Lebanon",
            "Lesotho",
            "Liberia",
            "Libyan Arab Jamahiriya",
            "Liechtenstein",
            "Lithuania",
            "Luxembourg",
            "Macau",
            "Madagascar",
            "Malawi",
            "Malaysia",
            "Maldives",
            "Mali",
            "Malta",
            "Marshall Islands",
            "Martinique",
            "Mauritania",
            "Mauritius",
            "Mayotte",
            "Mexico",
            "Micronesia, Federated States of",
            "Moldova, Republic of",
            "Monaco",
            "Mongolia",
            "Montenegro",
            "Montserrat",
            "Morocco",
            "Mozambique",
            "Myanmar",
            "Namibia",
            "Nauru",
            "Nepal",
            "Netherlands",
            "Netherlands Antilles",
            "New Caledonia",
            "New Zealand",
            "Nicaragua",
            "Niger",
            "Nigeria",
            "Niue",
            "Norfolk Island",
            "North Korea",
            "Northern Mariana Islands",
            "Norway",
            "Oman",
            "Pakistan",
            "Palau",
            "Palestinian Territory, Occupied",
            "Panama",
            "Papua New Guinea",
            "Paraguay",
            "Peru",
            "Philippines",
            "Pitcairn",
            "Poland",
            "Portugal",
            "Puerto Rico",
            "Qatar",
            "Reunion",
            "Romania",
            "Russian Federation",
            "Rwanda",
            "Saint Kitts and Nevis",
            "Saint Lucia",
            "Saint Vincent and the Grenadines",
            "Samoa",
            "San Marino",
            "Sao Tome and Principe",
            "Saudi Arabia",
            "Senegal",
            "Serbia",
            "Seychelles",
            "Sierra Leone",
            "Singapore",
            "Slovak Republic",
            "Slovenia",
            "Solomon Islands",
            "Somalia",
            "South Africa",
            "South Georgia & South Sandwich Islands",
            "South Sudan",
            "Spain",
            "Sri Lanka",
            "Sudan",
            "Suriname",
            "Svalbard and Jan Mayen Islands",
            "Sweden",
            "Switzerland",
            "Syrian Arab Republic",
            "Taiwan",
            "Tajikistan",
            "Tanzania",
            "Thailand",
            "Togo",
            "Tokelau",
            "Tonga",
            "Trinidad and Tobago",
            "Tunisia",
            "Turkey",
            "Turkmenistan",
            "Turks and Caicos Islands",
            "Tuvalu",
            "Uganda",
            "Ukraine",
            "United Arab Emirates",
            "United Kingdom",
            "United States",
            "Uruguay",
            "Uzbekistan",
            "Vanuatu",
            "Venezuela",
            "Viet Nam",
            "Wallis and Futuna Islands",
            "Western Sahara",
            "Yemen",
            "Zambia",
            "Zimbabwe"
        ];
    }

    public function verifyOTP($otp, $userId, $transaction_id){

        // Select the OTP expiration time
        $stmt = $this->prepare("SELECT tbl_otp_expire FROM ".TB_USER." WHERE tbl_id = :user_id AND tbl_otp = :otp");
        $stmt->bindParam(':otp', $otp);
        $stmt->bindParam(':user_id', $userId);
        $stmt->execute();

        if ($stmt->rowCount() > 0) {
            $row = $stmt->fetch();
            $otpExpireTime = $row['tbl_otp_expire'];

            // Check if the OTP has expired
            $currentTime = date('Y-m-d H:i:s');
            if ($otpExpireTime !== null && $currentTime > $otpExpireTime) {
                return array(
                    'status' => false,
                    'message' => 'OTP has expired. Please request a new one.'
                );
            }

            // OTP is still valid, update the transactions table
            $stmtUpdateTransaction = $this->prepare("UPDATE transactions SET currentlyRequired = 'cot' WHERE user_id = :user_id AND transaction_id = :transaction_id");
            $stmtUpdateTransaction->bindParam(':user_id', $userId);
            $stmtUpdateTransaction->bindParam(':transaction_id', $transaction_id);
            $stmtUpdateTransaction->execute();

            if ($stmtUpdateTransaction->rowCount() > 0) {

                // Clear the OTP and its expiration from the user table
                $stmtClearOTP = $this->prepare("UPDATE ".TB_USER." SET tbl_otp_expire = NULL, tbl_otp = NULL WHERE tbl_id = :user_id");
                $stmtClearOTP->bindParam(':user_id', $userId);
                $stmtClearOTP->execute();

                return array(
                    'status' => true,
                    'message' => 'OTP verified successfully. You can proceed to the next step.',
                    "transaction_id"=>$transaction_id
                );
            } else {
                return array(
                    'status' => false,
                    'message' => 'Failed to update the transaction. Please try again.'
                );
            }

        } else {
            return array(
                'status' => false,
                'message' => 'Invalid OTP.'
            );
        }
    }

    function resendOTP($userId){
        global $info;
        $stmt = $this->prepare("SELECT email FROM user_emails WHERE user_id =:user_id AND primary_email = 1");
        $stmt->bindParam(":user_id", $userId);
        $stmt->execute();
        if($stmt->rowCount()>0){
            $userEmail = $stmt->fetchColumn();


            $stmtSent = $this->prepare("UPDATE ".TB_USER." SET tbl_otp_expire = :expiry, tbl_otp = :otp WHERE tbl_id = :user_id");

            $number = $this->generateNumber(4, 'number'); // Generate a 4-digit OTP
            $expiry = date('Y-m-d H:i:s', strtotime('+1 hour')); // Set OTP expiration time to 1 hour from now

            $stmtSent->bindParam(":user_id", $userId);
            $stmtSent->bindParam(":otp", $number);
            $stmtSent->bindParam(":expiry", $expiry);
            $stmtSent->execute();

            //$userEmail['email'];

            $subject = "Your OTP Code";
            $message = "Dear user,\n\nYour One-Time Password (OTP) for your wire transfer is: " . $number . ".\n\nPlease use this code to proceed with your transaction. The OTP will expire in 1 hour.\n\nThank you!";
            $headers = "From: ".$info['email']."\r\n";
            $headers .= "Reply-To: ".$info['email']."\r\n";
            $headers .= "X-Mailer: PHP/" . phpversion();

            // Use PHP's mail function to send the OTP
//            if (mail($userEmail, $subject, $message, $headers)) {
            if ($this->sendEmail($userEmail, $subject, $message)) {
                // Email sent successfully
                return array(
                    "status" => true,
                    "message" => "OTP has been resent to your primary email."

                );
            } else {
                // Email sending failed
                return array(
                    "status" => false,
                    "message" => "Otp created, but failed to send OTP to your email, please try resending again"
                );
            }

        }else{
            return array(
                "status" => false,
                "message" => "Invalid request or user was not found"
            );
        }

    }

    public function verifyCOT($cot, $transaction_id, $user_id){
        $stmt = $this->prepare("SELECT * FROM ".TB_USER." WHERE tbl_cot = :cot AND tbl_id=:user_id");
        $stmt->bindParam(':user_id', $user_id);
        $stmt->bindParam(':cot', $cot);
        $stmt->execute();
        if($stmt->rowCount()>0){

            $tranStmt = $this->prepare("UPDATE transactions SET currentlyRequired = 'imf' WHERE transaction_id = :transaction_id AND user_id=:user_id");
            $tranStmt->bindParam(':user_id', $user_id);
            $tranStmt->bindParam(':transaction_id', $transaction_id);
            $tranStmt->execute();

            if($stmt->rowCount()>0){

                return array(
                    "status" => true,
                    "message" => "Thank you, your COT code has been verified successfully"
                );

            }else{

                return array(
                    "status" => false,
                    "message" => "Oops, something went wrong, we are unable to complete your request or COT is invalid "
                );

            }

        }else{
            return array(
                "status" => false,
                "message" => "Invalid COT Code"
            );
        }
    }

    function getLastDetails($description) {
        // Split the string by the '/' delimiter
        $parts = explode('/', $description);

        // Get the last two elements from the array
        $phoneNumber = $parts[count($parts) - 2] ?? null;
        $name = $parts[count($parts) - 1] ?? null;

        // Return the phone number and name as an array
        return [
            'phoneNumber' => $phoneNumber,
            'name' => $name
        ];
    }

    public function verifyIMF($imf, $transaction_id, $user_id, $wallet_number, $emails){

        global $info;

        $stmt = $this->prepare("SELECT tbl_first_name as first_name, tbl_last_name as last_name FROM ".TB_USER." WHERE tbl_imf = :imf AND tbl_id=:user_id");
        $stmt->bindParam(':user_id', $user_id);
        $stmt->bindParam(':imf', $imf);
//
        $stmt->execute();

        if($stmt->rowCount()>0){

            $senderData = $stmt->fetch(PDO::FETCH_ASSOC);

            $tranStmt = $this->prepare("UPDATE transactions SET currentlyRequired = 'done', status = 'completed' WHERE transaction_id = :transaction_id AND user_id=:user_id");
            $tranStmt->bindParam(':user_id', $user_id);
            $tranStmt->bindParam(':transaction_id', $transaction_id);
            $tranStmt->execute();

            if($tranStmt->rowCount()>0){



                $recordStmt = $this->prepare("SELECT * FROM transactions WHERE transaction_id = :transaction_id AND user_id=:user_id");
                $recordStmt->bindParam(':user_id', $user_id);
                $recordStmt->bindParam(':transaction_id', $transaction_id);
                $recordStmt->execute();
                $transData = $recordStmt->fetch(PDO::FETCH_ASSOC);


                $debitStmt = $this->prepare("UPDATE wallet SET balance = (balance-:amount) WHERE wallet_number = :wallet_number AND user_id=:user_id");
                $debitStmt->bindParam(':user_id', $user_id);
                $debitStmt->bindParam(':wallet_number', $wallet_number);
                $amount = $transData['amount'];
                $debitStmt->bindParam(':amount', $amount);
                $debitStmt->execute();


                $details = $this->getLastDetails($transData['description']);

                //send email receipt

                $to = $this->getPrimaryEmail($emails); // Assuming you have the receiver's email in the $details array
                $subject = "Transaction Receipt - Transaction ID: " . $transData['transaction_id'];
                $headers = "From: no-reply@yourdomain.com\r\n"; // Change to your domain
                $headers .= "Reply-To: no-reply@yourdomain.com\r\n";
                $headers .= "Content-Type: text/html; charset=UTF-8\r\n";

                // Prepare email body
                $emailBody = "
                    <html>
                    <head>
                        <title>Transaction Receipt</title>
                    </head>
                    <body>
                        <h2>Transaction Receipt</h2>
                        <p>Dear {$senderData['first_name']},</p>
                        <p>Thank you for your transaction. Here are the details:</p>
                        <ul>
                            <li>Transaction ID: {$transData['transaction_id']}</li>
                            <li>Sender Name: {$senderData['first_name']} {$senderData['last_name']}</li>
                            <li>Sender Account Number: {$wallet_number}</li>
                            <li>Receiver Name: {$details['name']}</li>
                            <li>Receiver Account Number: {$details['phoneNumber']}</li>
                            <li>Amount: <span style='color:red'>-{$transData['amount']} <span style='text-transform: uppercase'>{$transData['currency']}</span></span></li>
                            <li>Purpose: {$transData['description']}</li>
                            <li>Date: {$transData['transaction_date']}</li>
                            <li>Bank: {$transData['bank_name']}</li>
                        </ul>
                        <p>If you have any questions, feel free to contact us.</p>
                        <p>Best Regards,<br>{$info['name']}</p>
                         <footer>
                            <p><strong>Disclaimer:</strong> This email and its contents are for informational purposes only. The transaction details are based on records from our system at the time of processing. If you believe this email was sent in error, please contact our support team immediately.</p>
                        </footer>
                    </body>
                    </html>
                    ";


                //mail($to, $subject, $emailBody, $headers);

                $this->sendEmail($to, $subject, $emailBody);

                return array(
                    "status" => true,
                    "message" => "Thank you, your IMF code has been verified successfully",
                    'transaction_details' => [
                        'sender' => [
                            'account_number' => $wallet_number,
                            'name' => $senderData['first_name'].' '.$senderData['last_name'],
                            'user_id' => $transData['user_id'],
                        ],
                        'receiver' => [
                            'account_number' => $details['phoneNumber'],
                            'name' => $details['name'],
                            'user_id' => null,
                        ],
                        'amount' => $transData['amount'],
                        'currency' => $transData['currency'],
                        'purpose_of_transfer' => $transData['description'].' /'.$transData['transferOption'].'/'.$transData['transaction_type'],
                        'date' => $transData['created_at'],
                        'bank_name' => $transData['bank_name'],
                        'transaction_id' => $transData['transaction_id'],
                    ]
                );

            }else{

                return array(
                    "status" => false,
                    "message" => "Oops, something went wrong, we are unable to complete your request or IMF is invalid "
                );

            }


        }else{
            return array(
                "status" => false,
                "message" => "Invalid IMF Code"
            );

        }
    }

    public function getTransactionDetailsForVerification($transactionId, $userId){
        $stmt = $this->prepare("SELECT currentlyRequired as xcode FROM transactions WHERE user_id =:user AND transaction_id =:tId AND status = 'pending' AND (currentlyRequired = 'cot' OR currentlyRequired = 'imf' OR currentlyRequired = 'otp')");
        $stmt->bindParam(':tId', $transactionId);
        $stmt->bindParam(':user', $userId);
        $stmt->execute();
        return $stmt->fetch(PDO::FETCH_ASSOC);
    }

    function maskNumber($number) {
        // Convert the number to a string to handle it easily
        $numberStr = strval($number);
        // Get the length of the number
        $length = strlen($numberStr);

        // Ensure the number has at least 4 digits
        if ($length <= 4) {
            return $numberStr; // No masking needed for numbers with 4 or fewer digits
        }

        // Mask all but the last 4 digits
        $masked = str_repeat('*', $length - 4) . substr($numberStr, -4);

        return $masked;
    }

    public function transferFunds($receiverAccount, $currency, $receiverName, $amount, $purposeOfTransfer, $senderTransactionPINCode, $sendAccount, $senderId, $receiverId, $senderEmail) {
        // Start transaction
        $this->beginTransaction();

        global $info;
        $ourBank = $info['name'];

        try {
            // First, verify sender's transaction PIN using $senderTransactionPINCode and $senderId
            $stmt = $this->prepare("SELECT * FROM ".TB_USER." WHERE tbl_id = :sender AND tbl_transaction_pin = :pin");
            $stmt->bindParam("sender", $senderId);
            $stmt->bindParam("pin", $senderTransactionPINCode);
            $stmt->execute();
            $sentBy = $stmt->fetch(PDO::FETCH_ASSOC);

            if($stmt->rowCount() > 0) {
                // Debit the sender
                $stmtDebit = $this->prepare("UPDATE ".TB_USER_WALLET." SET balance = (balance - :amount) WHERE wallet_number = :senderAccount AND wallet = :currency AND user_id = :sender");
                $stmtDebit->bindParam('senderAccount', $sendAccount);
                $stmtDebit->bindParam("sender", $senderId);
                $stmtDebit->bindParam('currency', $currency);
                $stmtDebit->bindParam('amount', $amount);
                $stmtDebit->execute();


                if($stmtDebit->rowCount() > 0) {
                    // Credit the receiver
                    $stmtCredit = $this->prepare("UPDATE ".TB_USER_WALLET." SET balance = (balance + :amount) WHERE wallet_number = :receiver AND wallet = :currency AND user_id = :receiverId");
                    $stmtCredit->bindParam('receiver', $receiverAccount);
                    $stmtCredit->bindParam('receiverId', $receiverId);
                    $stmtCredit->bindParam('currency', $currency);
                    $stmtCredit->bindParam('amount', $amount);
                    $stmtCredit->execute();


                    if($stmtCredit->rowCount() > 0) {
                        // Generate a unique transaction ID
                        $transactionId = 'TRX' . time() . rand(1000, 9999);

                        // Create transaction history for the sender
                        $stmtTransactionSender = $this->prepare("INSERT INTO transactions 
                        (user_id, transaction_date, transaction_type, amount, currency, description, status, bank_name, transaction_id,transferOption,created_at) 
                        VALUES (:senderId, NOW(), 'withdrawal', :amount, :currency, :description, 'completed', :ourBank, :transaction_id, 'SameBank',:created_at)");
                        $stmtTransactionSender->bindParam('ourBank', $ourBank);
                        $stmtTransactionSender->bindParam('senderId', $senderId);
                        $stmtTransactionSender->bindParam('amount', $amount);
                        $stmtTransactionSender->bindParam('currency', $currency);
                        $d = $purposeOfTransfer. ' / sent to '.$receiverName;
                        $stmtTransactionSender->bindParam('description', $d);
                        $stmtTransactionSender->bindParam('transaction_id', $transactionId);
                        $created_at = date('Y-m-d H:i:s');
                        $stmtTransactionSender->bindParam(":created_at", $created_at);
                        $stmtTransactionSender->execute();

                        // Create transaction history for the receiver
                        $stmtTransactionReceiver = $this->prepare("INSERT INTO transactions 
                        (user_id, transaction_date, transaction_type, amount, currency, description, status, bank_name, transaction_id, transferOption, created_at) 
                        VALUES (:receiverId, NOW(), 'credit', :amount, :currency, :description, 'completed', :ourBank, :transaction_id, 'SameBank', :created_at)");
                        $stmtTransactionReceiver->bindParam('ourBank', $ourBank);
                        $stmtTransactionReceiver->bindParam('receiverId', $receiverId);
                        $stmtTransactionReceiver->bindParam('amount', $amount);
                        $stmtTransactionReceiver->bindParam('currency', $currency);
                        $desc = 'Received from ' . $sentBy['tbl_first_name'] .' '.$sentBy['tbl_last_name'];
                        $stmtTransactionReceiver->bindParam('description', $desc);
                        $stmtTransactionReceiver->bindParam('transaction_id', $transactionId);
                        $created_at = date('Y-m-d H:i:s');
                        $stmtTransactionReceiver->bindParam(":created_at", $created_at);
                        $stmtTransactionReceiver->execute();

                        // Fetch sender details for the receipt
                        $stmtSenderDetails = $this->prepare("SELECT tbl_first_name AS firstName, tbl_last_name AS lastName FROM ".TB_USER." WHERE tbl_id = :senderId");
                        $stmtSenderDetails->bindParam('senderId', $senderId);
                        $stmtSenderDetails->execute();
                        $senderDetails = $stmtSenderDetails->fetch(PDO::FETCH_ASSOC);

                        // Fetch receiver details for the receipt
                        $stmtReceiverDetails = $this->prepare("SELECT tbl_first_name AS firstName, tbl_last_name AS lastName, email FROM ".TB_USER." left join user_emails on tbl_id=user_id WHERE tbl_id = :receiverId and primary_email = 1");
                        $stmtReceiverDetails->bindParam('receiverId', $receiverId);
                        $stmtReceiverDetails->execute();
                        $receiverDetails = $stmtReceiverDetails->fetch(PDO::FETCH_ASSOC);

                        // Commit the transaction
                        $this->commit();


                        // Send debit email to sender
                        $senderEmailBody = "
                        <html>
                        <body>
                            <h2>Debit Alert</h2>
                            <p>Dear {$senderDetails['firstName']},</p>
                            <p>Your account ending in {$this->maskNumber($sendAccount)} has been debited <span style='color:red'>-{$amount}{strtoupper($currency)}</span> for the purpose of '{$purposeOfTransfer}'.</p>
                            <p>Transaction ID: {$transactionId}</p>
                            <p>Recipient: {$receiverDetails['firstName']} {$receiverDetails['lastName']} ({$receiverAccount})</p>
                            <p>Thank you for using our services.</p>
                        </body>
                        </html>";
                        $this->sendEmail($senderEmail, "Debit Transaction Notification", $senderEmailBody);

                        // Send credit email to receiver
                        $receiverEmailBody = "
                        <html>
                        <body>
                            <h2>Credit Alert</h2>
                            <p>Dear {$receiverDetails['firstName']},</p>
                            <p>Your account ending in {$this->maskNumber($receiverAccount)} has been credited with <span style='color:lime'>+{$amount}{strtoupper($currency)}</span>.</p>
                            <p>Transaction ID: {$transactionId}</p>
                            <p>Sender: {$sentBy['tbl_first_name']} {$sentBy['tbl_last_name']} ({$sendAccount})</p>
                            <p>Thank you for using our services.</p>
                        </body>
                        </html>";
                        $this->sendEmail($receiverDetails['email'], "Credit Transaction Notification", $receiverEmailBody);


                        // Return success response with detailed info for receipt
                        return [
                            'status' => true,
                            'message' => 'Transfer completed successfully.',
                            'transaction_details' => [
                                'sender' => [
                                    'account_number' => $sendAccount,
                                    'name' => $senderDetails['firstName'] . ' ' . $senderDetails['lastName'],
                                    'user_id' => $senderId,
                                ],
                                'receiver' => [
                                    'account_number' => $receiverAccount,
                                    'name' => $receiverDetails['firstName'] . ' ' . $receiverDetails['lastName'],
                                    'user_id' => $receiverId,
                                ],
                                'amount' => $amount,
                                'currency' => $currency,
                                'purpose_of_transfer' => $purposeOfTransfer,
                                'date' => date("Y-m-d H:i:s"),
                                'bank_name' => $ourBank,
                                'transaction_id' => $transactionId,
                            ]
                        ];

                    } else {
                        // Roll back the transaction if crediting fails
                        $this->rollBack();
                        return [
                            'status' => false,
                            'message' => 'Failed to credit the receiver\'s account.'
                        ];
                    }

                } else {
                    // Roll back the transaction if debiting fails
                    $this->rollBack();
                    return [
                        'status' => false,
                        'message' => 'Failed to debit the sender\'s account.'
                    ];
                }

            } else {
                return [
                    'status' => false,
                    'message' => 'Invalid transaction PIN.'
                ];
            }

        } catch (Exception $e) {
            // Roll back in case of any error
            $this->rollBack();
            return [
                'status' => false,
                'message' => 'Transaction failed: ' . $e->getMessage()
            ];
        }
    }

    function fetchLanguages() {
        $stmt = $this->prepare("SELECT short_lan FROM system_language WHERE status = 1");
        $stmt->execute();

        // Fetch all short_lan values
        $languages = $stmt->fetchAll(PDO::FETCH_COLUMN);

        // Join the array into a string with commas
        return implode(',', $languages);
    }

    function defaultLanguage() {
        $stmt = $this->prepare("SELECT short_lan FROM system_language WHERE status = 1 and isDefault = 1");
        $stmt->execute();
        return $stmt->fetch(PDO::FETCH_COLUMN);
    }

    function systemLanguages() {
        $stmt = $this->prepare("SELECT short_lan, language, status, isDefault FROM system_language");
        $stmt->execute();
        $array = [];
        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
            $array[] = $row;
        }
        return $array;
    }

    function handleLanguageEnabling($languageId, $status){
        $stmt = $this->prepare("UPDATE system_language set status = :status WHERE short_lan =:short_lan");
        $stmt->bindParam(":status", $status);
        $stmt->bindParam(":short_lan", $languageId);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "status"=>true,
                "message"=>"Language enabled"
            );
        }else{
            return array(
                "status"=>false,
                "message"=>"Could not enabled language, try again"
            );
        }
    }

    function handleDefaultLanguageEnabling($languageId, $status) {
        // Start by ensuring only one language is marked as default
        $this->beginTransaction(); // Start transaction to ensure atomic operation

        try {
            // Check if there is any other language already marked as default
            $stmt = $this->prepare("UPDATE system_language SET isDefault = 0 WHERE isDefault = 1");
            $stmt->execute(); // Reset any other default language

            // Now set the current language as default and update status
            $stmt = $this->prepare("UPDATE system_language SET status = :status, isDefault = :isDefault WHERE short_lan = :short_lan");
            $stmt->bindParam(":status", $status);
            $stmt->bindParam(":isDefault", $status); // Mark this language as default (1) if the status is 1
            $stmt->bindParam(":short_lan", $languageId);
            $stmt->execute();

            // Commit the transaction if everything was successful
            if ($stmt->rowCount() > 0) {
                $this->commit(); // Commit the transaction
                return array(
                    "status" => true,
                    "message" => "Language enabled and set as default."
                );
            } else {
                $this->rollBack(); // Rollback in case of failure
                return array(
                    "status" => false,
                    "message" => "Could not enable language, try again."
                );
            }
        } catch (Exception $e) {
            // In case of any error, rollback the transaction
            $this->rollBack();
            return array(
                "status" => false,
                "message" => "An error occurred: " . $e->getMessage()
            );
        }
    }

    function formatAccountStatus2($status)
    {
        $statusMap = [
            'Active' => '<span class="border rounded-pill fw-500 text-2 px-3 py-1 ms-2" style="color: #9edd05"><i class="fas fa-check-circle"></i> Active</span>',
            'Dormant' => '<span class="border text-warning rounded-pill fw-500 text-2 px-3 py-1 ms-2"><i class="fas fa-clock"></i> Dormant</span>',
            'Frozen' => '<span class="border text-info rounded-pill fw-500 text-2 px-3 py-1 ms-2"><i class="fas fa-snowflake"></i> Frozen</span>',
            'Locked' => '<span class="border text-secondary rounded-pill fw-500 text-2 px-3 py-1 ms-2"><i class="fas fa-lock"></i> Locked</span>',
            'Banned' => '<span class="border text-danger rounded-pill fw-500 text-2 px-3 py-1 ms-2"><i class="fas fa-ban"></i> Banned</span>',
            'Under-Review' => '<span class="border text-primary rounded-pill fw-500 text-2 px-3 py-1 ms-2"><i class="fas fa-search"></i> Under Review</span>',
            'Archived' => '<span class="border text-dark rounded-pill fw-500 text-2 px-3 py-1 ms-2"><i class="fas fa-archive"></i> Archived</span>',
        ];

        // Default to a generic status if not recognized
        return $statusMap[$status] ?? '<span class="bg-light text-dark rounded-pill d-inline-block px-2 mb-0"><i class="fas fa-question-circle"></i> Unknown Status</span>';
    }

    // Function to calculate the completion percentage
    function calculateCompletionPercentage(array $userDetails): string {

        $profileFieldsConfig = [
            'firstName' => true,
            'lastName' => true,
            'avatar' => true, // Optional field
            'dob' => true, // Optional field
            'address' => true, // Optional field
            'city' => true, // Optional field
            'state' => true, // Optional field
            'zipcode' => true, // Optional field
            'country' => true, // Optional field
            'user_language' => true,
            'time_zone' => false, // Optional field
            'account_status' => true,
            'emails' => true, // Will be checked separately
            'wallets' => true  // Will be checked separately
        ];

        $totalFields = count($profileFieldsConfig);
        $providedCount = 0;

        foreach ($profileFieldsConfig as $field => $isRequired) {
            if (isset($userDetails[$field]) && !empty($userDetails[$field])) {
                if ($field === 'emails' && is_array($userDetails[$field])) {
                    foreach ($userDetails[$field] as $email) {
                        if (!empty($email['email'])) {
                            $providedCount++;
                        }
                    }
                } elseif ($field === 'wallets' && is_array($userDetails[$field])) {
                    foreach ($userDetails[$field] as $wallet) {
                        if (!empty($wallet['wallet']) && isset($wallet['balance'])) {
                            $providedCount++;
                        }
                    }
                } else {
                    $providedCount++;
                }
            }
        }

        // Calculate completion percentage
        $completionPercentage = min(100, ($providedCount / ($totalFields + count($userDetails['emails']) + count($userDetails['wallets']))) * 100);

        return round($completionPercentage) . '%';
    }

    public function getUserTransactions($userId, $type = null, $dateRange = null) {
        $query = "SELECT * FROM transactions WHERE user_id = ?";
        $params = [$userId];

        if (!empty($type)) {
            $query .= " AND transaction_type = ?";
            $params[] = $type;
        }

        if (!empty($dateRange)) {
            // Split the date range into start and end dates
            list($startDate, $endDate) = explode(' - ', $dateRange);

            // Convert dates to YYYY-MM-DD format
            $startDate = date('Y-m-d', strtotime($startDate));
            $endDate = date('Y-m-d', strtotime($endDate)) . ' 23:59:59';

            // Add BETWEEN condition
            $query .= " AND created_at BETWEEN ? AND ?";
            $params[] = $startDate . ' 00:00:00';
            $params[] = $endDate;
        }

        // Append order clause at the end
        $query .= " ORDER BY created_at DESC";

        // Debug: Log the query and params
        //error_log("SQL Query: " . $query);
        //error_log("Parameters: " . json_encode($params));

        try {
            // Prepare and execute the statement
            $stmt = $this->prepare($query);
            $stmt->execute($params);
            return $stmt->fetchAll(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            // Log any potential PDO error
            //error_log("Database error: " . $e->getMessage());
            return [];
        }
    }

    function calculateTransactionTotals($transactions) {
        // Initialize totals for each currency
        $totals = [
            'usd' => 0,
            'euro' => 0,
            'pound' => 0,
        ];

        foreach ($transactions as $transaction) {
            $currency = strtolower($transaction['currency']); // Convert to lowercase for consistency
            $amount = (float)$transaction['amount'];

           // echo "Amount: {$amount}\n";

            // Add amount to corresponding currency total
            switch ($currency) {
                case 'usd':
                    $totals['usd'] += $amount;
                    break;
                case 'euro':
                    $totals['euro'] += $amount;
                    break;
                case 'pound':
                    $totals['pound'] += $amount;
                    break;
                // Optionally handle more currencies here
            }
        }

        return $totals;
    }

    public function saveAvatar($newFileName, $userId){
        $sql = "UPDATE ".TB_USER." SET tbl_avatar = :profile_image WHERE tbl_id = :user_id";
        $stmt = $this->prepare($sql);
        $stmt->bindParam(':profile_image', $newFileName);
        $stmt->bindParam(':user_id', $userId);
        $stmt->execute();
        return $stmt->rowCount() >0;
    }

    public function creteNewPin($pin, $userId){
        $stmt = $this->prepare("UPDATE ".TB_USER." SET tbl_transaction_pin = :pin WHERE tbl_id = :userid");
        $stmt->bindParam(":pin", $pin);
        $stmt->bindParam(":userid", $userId);
        $stmt->execute();
        if($stmt->rowCount()>0){
            return array(
                "status"=>true,
                "message"=>"Pin created successfully"
            );
        }else{
            return array(
                "status"=>false,
                "message"=>"Oops, something went wrong, try again"
            );
        }
    }

    public function messageContaner($children, $type){

        return $children ? '<div class="alert alert-'.$type.' alert-dismissible fade show" role="alert">
                        '.$children.'
                    </div>' : null;
    }

    public function bottomSheet($children, $close, $bottomSheet){
        $x = '<div class="bottom-sheet-overlay" id="bottomSheetOverlay"></div>

    <div class="bottom-sheet" id="'.$bottomSheet.'">
        <div class="bottom-sheet-header">
            <span class="bottom-sheet-handle"></span>
            <div class="close-btn"><button style="background-color: rgba(0,192,118,0);border-radius: 50px;border: 0;color: red;font-size: 34px" id="'.$close.'">&times;</button></div>
        </div>
        <div class="bottom-sheet-content">
        ' .$children.'
        </div>
    </div>';
        echo $x;
    }

    public function displayTransactions($transactions) {
        // Check if there are transactions
        if (empty($transactions)) {
            // Return the empty state message
            return '<div class="text-center py-4">
                    <div class="text-17 text-light my-3"><i class="fas fa-box-open"></i></div>
                    <h5 class="text-muted">No Data.</h5>
                    <p class="text-muted">Your recent history appears here...</p>
                </div>';
        }

        $html = '<div class="transaction-list">';

        foreach ($transactions as $transaction) {
            // Format the date
            $date = new DateTime($transaction['created_at']);
            $formattedDate = $date->format('d');
            $formattedMonth = strtoupper($date->format('M'));

            // Determine the transaction status icon and color
            switch ($transaction['status']) {
                case 'completed':
                    $statusIcon = '<i class="fas fa-check-circle"></i>';
                    $statusClass = 'text-success';
                    $statusTooltip = 'Completed';
                    break;
                case 'pending':
                    $statusIcon = '<i class="fas fa-ellipsis-h"></i>';
                    $statusClass = 'text-warning';
                    $statusTooltip = 'In Progress';
                    break;
                case 'cancelled':
                    $statusIcon = '<i class="fas fa-times-circle"></i>';
                    $statusClass = 'text-danger';
                    $statusTooltip = 'Cancelled';
                    break;
                default:
                    $statusIcon = '<i class="fas fa-times-circle"></i>';
                    $statusClass = 'text-danger';
                    $statusTooltip = 'Failed';
            }

            // Add to the HTML output
            $html .= '
        <div class="transaction-item px-4 py-3" onclick="getTransactionDetails(\'' . htmlspecialchars($transaction['transaction_id']) . '\')">
            <div class="row align-items-center flex-row">
                <div class="col-2 col-sm-1 text-center">
                    <span class="d-block text-4 fw-300">' . $formattedDate . '</span>
                    <span class="d-block text-1 fw-300 text-uppercase">' . $formattedMonth . '</span>
                </div>
                <div class="col col-sm-7">
                    <span class="d-block text-4">' . htmlspecialchars($transaction['bank_name'] ?? $transaction['description']) . '</span>
                    <span class="text-muted">' . htmlspecialchars($transaction['description']) . '</span>
                </div>
                <div class="col-auto col-sm-2 d-none d-sm-block text-center text-3">
                    <span class="' . $statusClass . '" data-bs-toggle="tooltip" title="' . $statusTooltip . '">' . $statusIcon . '</span>
                </div>
                <div class="col-3 col-sm-2 text-end text-4">
                    <span class="text-nowrap">' . ($transaction['transaction_type'] === 'withdrawal' ? '- ' : ' + ') .$this->returnCurrencySymbol($transaction['currency']). number_format($transaction['amount']) . '</span>
                  
                </div>
            </div>
        </div>';
        }
//        <span class="text-2 text-uppercase">(' . htmlspecialchars($transaction['currency']) . ')</span>
        $html .= '</div>';
        return $html;
    }

    public function returnCurrencySymbol($currency){
        if ($currency == "pound") {
            $symbol = "£";
        }else if($currency == "usd"){
            $symbol = "$";
        }else if($currency == "euro"){
            $symbol = "€";
        }else{
            $symbol = "$";
        }
        return $symbol;
    }

    public function getTransactionDetail($transactionId, $userId) {
        $query = "SELECT id, user_id, transaction_date, transaction_type, amount, currency, description, status, bank_name, transaction_id, transferOption, currentlyRequired, created_at FROM transactions WHERE transaction_id = ? AND user_id = ? ";
        $stmt = $this->prepare($query);
        $stmt->execute([$transactionId, $userId]);
        return $stmt->fetch(PDO::FETCH_ASSOC);
    }

    public function getTransactionDetailsHandler($transactionId, $userId) {
        $transaction = $this->getTransactionDetail($transactionId, $userId);

        // Prepare the response
        if ($transaction) {
            $response = [
                'status' => true,
                'data' => [
                    'bank_name' => $transaction['bank_name'],
                    'amount' => $transaction['amount'],
                    'currency' => $transaction['currency'],
                    'transaction_date' => $transaction['created_at'],
                    'description' => $transaction['description'],
                    'transaction_type' => $transaction['transaction_type'],
                    'transaction_id' => $transaction['transaction_id'],
                    'status' => $transaction['status'],
                    'user_id' => $transaction['user_id'],
                    'currentlyRequired' => $transaction['currentlyRequired'],
                    'transferOption' => $transaction['transferOption'],
                    'created_at' => $transaction['created_at']
                ]
            ];
        } else {
            $response = [
                'status' => false,
                'message' => 'Transaction not found. ID: ' . $transactionId
            ];
        }

        // Set content type header
//        header('Content-Type: application/json');
        return $response;
    }

    public function displayTransactionDetail($transaction) {
        $html = '<div id="transaction-detail" class="modal fade" role="dialog" aria-hidden="true">
                <div class="modal-dialog modal-dialog-centered transaction-details" role="document">
                    <div class="modal-content">
                        <div class="modal-body">
                            <div class="row g-0">
                                <div class="col-sm-5 d-flex justify-content-center bg-primary rounded-start py-4">
                                    <div class="my-auto text-center">
                                        <div class="text-17 text-white my-3"><i class="fas fa-building"></i></div>
                                        <h3 class="text-4 text-white fw-400 my-3">' . htmlspecialchars($transaction['bank_name'] ?? 'Transaction') . '</h3>
                                        <div class="text-8 fw-500 text-white my-4">$' . number_format($transaction['amount'], 2) . '</div>
                                        <p class="text-white">' . date('d F Y', strtotime($transaction['transaction_date'])) . '</p>
                                    </div>
                                </div>
                                <div class="col-sm-7">
                                    <h5 class="text-5 fw-400 m-3">Transaction Details
                                        <button type="button" class="btn-close text-2 float-end" data-bs-dismiss="modal" aria-label="Close"></button>
                                    </h5>
                                    <hr>
                                    <div class="px-3">
                                        <ul class="list-unstyled">
                                            <li class="mb-2">Payment Amount <span class="float-end text-3">$' . number_format($transaction['amount'], 2) . '</span></li>
                                            <li class="mb-2">Fee <span class="float-end text-3">- $0.00</span></li> <!-- Add fee logic if applicable -->
                                        </ul>
                                        <hr class="mb-2">
                                        <p class="d-flex align-items-center fw-500 mb-0">Total Amount <span class="text-3 ms-auto">$' . number_format($transaction['amount'], 2) . '</span></p>
                                        <hr class="mb-4 mt-2">
                                        <ul class="list-unstyled">
                                            <li class="fw-500">Paid By:</li>
                                            <li class="text-muted">' . htmlspecialchars($transaction['bank_name'] ?? 'N/A') . '</li>
                                        </ul>
                                        <ul class="list-unstyled">
                                            <li class="fw-500">Transaction ID:</li>
                                            <li class="text-muted">' . htmlspecialchars($transaction['transaction_id'] ?? 'N/A') . '</li>
                                        </ul>
                                        <ul class="list-unstyled">
                                            <li class="fw-500">Description:</li>
                                            <li class="text-muted">' . htmlspecialchars($transaction['description']) . '</li>
                                        </ul>
                                        <ul class="list-unstyled">
                                            <li class="fw-500">Status:</li>
                                            <li class="text-muted">' . htmlspecialchars(ucfirst($transaction['status'])) . '<span class="text-success text-3 ms-1"><i class="fas fa-check-circle"></i></span></li>
                                        </ul>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>';

        return $html;
    }

    public function addOtherBank($userId, $country, $bankName, $accountName, $accountNumber, $ifscCode, $swiftCode, $bankAddress, $accountType) {
        // Prepare the SQL statement
        $stmt = $this->prepare("INSERT INTO bank_accounts (user_id, country, bank_name, account_name, account_number, ifsc_code, swift_code, bank_address, account_type) 
                             VALUES (:user_id, :country, :bank_name, :account_name, :account_number, :ifsc_code, :swift_code, :bank_address, :account_type)");

        // Bind parameters
        $stmt->bindParam(':user_id', $userId, PDO::PARAM_INT);
        $stmt->bindParam(':country', $country, PDO::PARAM_STR);
        $stmt->bindParam(':bank_name', $bankName, PDO::PARAM_STR);
        $stmt->bindParam(':account_name', $accountName, PDO::PARAM_STR);
        $stmt->bindParam(':account_number', $accountNumber, PDO::PARAM_STR);
        $stmt->bindParam(':ifsc_code', $ifscCode, PDO::PARAM_STR);
        $stmt->bindParam(':swift_code', $swiftCode, PDO::PARAM_STR);
        $stmt->bindParam(':bank_address', $bankAddress, PDO::PARAM_STR);
        $stmt->bindParam(':account_type', $accountType, PDO::PARAM_STR);

        // Execute the statement
        return $stmt->execute();
    }

    public function getOtherBanks($user_id) {
        $stmt = $this->prepare("SELECT country, bank_name, account_name, account_number, ifsc_code, swift_code, bank_address, account_type, created_at, primary_bank, status FROM bank_accounts WHERE user_id = :user_id");
        $stmt->bindParam(':user_id', $user_id); // Ensure the parameter name has a colon
        $stmt->execute();

        // Fetch all results as an associative array
        return $stmt->fetchAll(PDO::FETCH_ASSOC); // Return the array of bank accounts
    }

    public function bankAccountExists($userId, $accountNumber) {
        // Prepare the SQL statement
        $stmt = $this->prepare("SELECT COUNT(*) FROM bank_accounts WHERE account_number = :account_number AND user_id = :user_id");

        // Bind parameters
        $stmt->bindParam(':account_number', $accountNumber, PDO::PARAM_STR);
        $stmt->bindParam(':user_id', $userId, PDO::PARAM_INT);

        // Execute the statement
        $stmt->execute();

        // Fetch the result
        $count = $stmt->fetchColumn();

        // Return true if the account exists, false otherwise
        return $count > 0;
    }

    public function generateNumber(int $length, $type = null): string
    {
        if(strtolower($type) =='number'){
            $characters = "54788903920385487827382734837847361831383483548593092399284837573527378283937463477298323874637239483647584722732859735635674827328398293726336473849383427362763723849839488362375235243";
        }else{
            $characters = "4XZC4CXC83GJA3S4ASDd974ASA938DSD938F928H94SB9dSDD6SDSd30X92SD8SD09029HF323DKS749849K049340D30SF8DF1M089129F0NM6SDS11BNM010SD1A8SDS5B8VBS9VDD30SD237SD8233D100VB5VB0SD1FDF30G2394ZC6VCB9834J8734";
        }

        $charactersLength = strlen($characters);
        $randomString = "";
        for($i = 0; $i < $length; $i++ ){
            $randomString .=$characters[rand(0, $charactersLength -1)];
        }
        return $randomString;
    }

    public function notificationsList($userId){
        $stmt = $this->prepare("SELECT id, message, notification_date, status, created_at, message_title FROM notifications WHERE user_id = :user_id");
        $stmt->bindParam(':user_id', $userId, PDO::PARAM_INT);
        $stmt->execute();

        // Fetch all results as an associative array
        return $stmt->fetchAll(PDO::FETCH_ASSOC); // Return the array of bank accounts
    }

    function getPrimaryEmail($emails) {
        foreach ($emails as $email) {
            if (isset($email['primary_email']) && $email['primary_email'] == 1) {
                return $email['email'];
            }
        }
        return 'N/A'; // Return null if no primary email is found
    }

    function getPrimaryPhone($phones) {
        foreach ($phones as $phone) {
            if (isset($phone['primary_phone']) && $phone['primary_phone'] == 1) {
                return $phone['phone'];
            }
        }
        return 'N/A'; // Return null if no primary email is found
    }

    function generateCardNumber($cardType = 'visa'): string
    {
        // Determine the starting digits and length based on the card type
        switch (strtolower($cardType)) {
            case 'mastercard':
                $startDigits = rand(51, 55); // Mastercard starts with 51-55
                $length = 16;
                break;
            case 'visa':
                $startDigits = 4; // Visa starts with 4
                $length = 16;
                break;
            case 'amex':
                $startDigits = rand(34, 37); // Amex starts with 34 or 37
                $length = 15;
                break;
            case 'discover':
                $startDigits = 6011; // Discover starts with 6011
                $length = 16;
                break;
            default:
                // Default to Visa card if no valid type is provided
                $startDigits = 4;
                $length = 16;
        }

        // Generate the remaining digits
        $remainingLength = $length - strlen($startDigits);
        $remainingDigits = '';

        for ($i = 0; $i < $remainingLength; $i++) {
            $remainingDigits .= rand(0, 9);
        }

        // Combine the starting digits with the remaining digits
        $cardNumber = $startDigits . $remainingDigits;

        // Format as 'number-number-number'
        return substr($cardNumber, 0, 4) . '-' .
            substr($cardNumber, 4, 6) . '-' .
            substr($cardNumber, 10);
    }

    function maskCardNumber($cardNumber) {
        // Remove any non-numeric characters (just in case)
        $cleanNumber = preg_replace('/\D/', '', $cardNumber);

        // Ensure the number has at least 16 digits (you can adjust this based on your card number length)
        if (strlen($cleanNumber) < 16) {
            return "Invalid card number";
        }

        // Mask the first 12 digits, leaving the last 4 visible
        return 'XXXX-XXXX-XXXX-' . substr($cleanNumber, -4);
    }

    function removeHyphens($cardNumber) {
        // Use preg_replace to remove all hyphens
        return preg_replace('/-/', '', $cardNumber);
    }

    function generateAccountNumber_old($country) {
        // Define account number formats based on country
        $formats = [
            "Aaland Islands" => 10,
            "Afghanistan" => 10,
            "Albania" => 10,
            "Algeria" => 10,
            "American Samoa" => 11,
            "Andorra" => 12,
            "Angola" => 12,
            "Antarctica" => 12,
            "Antigua and Barbuda" => 10,
            "Argentina" => 10,
            "Armenia" => 10,
            "Aruba" => 10,
            "Ascension Island (British)" => 10,
            "Australia" => 9,
            "Austria" => 12,
            "Azerbaijan" => 12,
            "Bahamas" => 12,
            "Bahrain" => 8,
            "Bangladesh" => 10,
            "Barbados" => 10,
            "Belarus" => 28,
            "Belgium" => 10,
            "Belize" => 28,
            "Benin" => 12,
            "Bermuda" => 12,
            "Bhutan" => 10,
            "Bolivia" => 24,
            "Bonaire, Sint Eustatius and Saba" => 10,
            "Bosnia and Herzegovina" => 20,
            "Botswana" => 12,
            "Bouvet Island" => 10,
            "Brazil" => 10,
            "British Indian Ocean Territory" => 10,
            "Brunei Darussalam" => 14,
            "Bulgaria" => 10,
            "Burkina Faso" => 12,
            "Burundi" => 12,
            "Cambodia" => 9,
            "Cameroon" => 12,
            "Canada" => 12,
            "Canary Islands" => 10,
            "Cape Verde" => 10,
            "Cayman Islands" => 8,
            "Central African Republic" => 13,
            "Chad" => 12,
            "Chile" => 9,
            "China" => 19,
            "Christmas Island" => 10,
            "Cocos (Keeling) Islands" => 10,
            "Colombia" => 14,
            "Comoros" => 12,
            "Congo" => 12,
            "Cook Islands" => 10,
            "Costa Rica" => 10,
            "Cote DIvoire" => 12,
            "Croatia" => 11,
            "Cuba" => 16,
            "Curacao" => 10,
            "Cyprus" => 28,
            "Czech Republic" => 16,
            "Democratic Republic of Congo" => 12,
            "Denmark" => 10,
            "Djibouti" => 27,
            "Dominica" => 10,
            "Dominican Republic" => 11,
            "East Timor" => 9,
            "Ecuador" => 10,
            "Egypt" => 12,
            "El Salvador" => 14,
            "Equatorial Guinea" => 12,
            "Eritrea" => 12,
            "Estonia" => 20,
            "Ethiopia" => 12,
            "Falkland Islands (Malvinas)" => 10,
            "Faroe Islands" => 10,
            "Fiji" => 12,
            "Finland" => 14,
            "France, Metropolitan" => 10,
            "French Guiana" => 11,
            "French Polynesia" => 10,
            "French Southern Territories" => 10,
            "FYROM" => 19,
            "Gabon" => 12,
            "Gambia" => 13,
            "Georgia" => 12,
            "Germany" => 10,
            "Ghana" => 10,
            "Gibraltar" => 8,
            "Greece" => 12,
            "Greenland" => 10,
            "Grenada" => 10,
            "Guadeloupe" => 10,
            "Guam" => 10,
            "Guatemala" => 8,
            "Guernsey" => 10,
            "Guinea" => 12,
            "Guinea-Bissau" => 10,
            "Guyana" => 10,
            "Haiti" => 8,
            "Heard and Mc Donald Islands" => 10,
            "Honduras" => 14,
            "Hong Kong" => 12,
            "Hungary" => 16,
            "Iceland" => 12,
            "India" => 11,
            "Indonesia" => 10,
            "Iran (Islamic Republic of)" => 16,
            "Iraq" => 14,
            "Ireland" => 8,
            "Isle of Man" => 10,
            "Israel" => 9,
            "Italy" => 10,
            "Jamaica" => 10,
            "Japan" => 7,
            "Jersey" => 10,
            "Jordan" => 10,
            "Kazakhstan" => 12,
            "Kenya" => 12,
            "Kiribati" => 10,
            "Korea, Republic of" => 12,
            "Kosovo, Republic of" => 10,
            "Kuwait" => 30,
            "Kyrgyzstan" => 20,
            "Lao Peoples Democratic Republic" => 10,
            "Latvia" => 21,
            "Lebanon" => 12,
            "Lesotho" => 10,
            "Liberia" => 12,
            "Libyan Arab Jamahiriya" => 12,
            "Liechtenstein" => 12,
            "Lithuania" => 20,
            "Luxembourg" => 14,
            "Macau" => 15,
            "Madagascar" => 12,
            "Malawi" => 10,
            "Malaysia" => 12,
            "Maldives" => 16,
            "Mali" => 12,
            "Malta" => 23,
            "Marshall Islands" => 10,
            "Martinique" => 10,
            "Mauritania" => 14,
            "Mauritius" => 14,
            "Mayotte" => 12,
            "Mexico" => 10,
            "Micronesia, Federated States of" => 10,
            "Moldova, Republic of" => 12,
            "Monaco" => 10,
            "Mongolia" => 16,
            "Montenegro" => 13,
            "Montserrat" => 10,
            "Morocco" => 12,
            "Mozambique" => 12,
            "Myanmar" => 16,
            "Namibia" => 12,
            "Nauru" => 10,
            "Nepal" => 10,
            "Netherlands" => 10,
            "Netherlands Antilles" => 10,
            "New Caledonia" => 10,
            "New Zealand" => 16,
            "Nicaragua" => 12,
            "Niger" => 13,
            "Nigeria" => 10,
            "Niue" => 9,
            "Norfolk Island" => 10,
            "North Korea" => 10,
            "Northern Mariana Islands" => 10,
            "Norway" => 11,
            "Oman" => 23,
            "Pakistan" => 16,
            "Palau" => 12,
            "Palestinian Territory, Occupied" => 11,
            "Panama" => 8,
            "Papua New Guinea" => 10,
            "Paraguay" => 20,
            "Peru" => 12,
            "Philippines" => 10,
            "Pitcairn" => 10,
            "Poland" => 26,
            "Portugal" => 22,
            "Puerto Rico" => 10,
            "Qatar" => 29,
            "Reunion" => 10,
            "Romania" => 24,
            "Russian Federation" => 20,
            "Rwanda" => 10,
            "Saint Kitts and Nevis" => 10,
            "Saint Lucia" => 10,
            "Saint Vincent and the Grenadines" => 10,
            "Samoa" => 10,
            "San Marino" => 27,
            "Sao Tome and Principe" => 25,
            "Saudi Arabia" => 20,
            "Senegal" => 12,
            "Serbia" => 20,
            "Seychelles" => 12,
            "Sierra Leone" => 10,
            "Singapore" => 10,
            "Slovak Republic" => 12,
            "Slovenia" => 16,
            "Solomon Islands" => 10,
            "Somalia" => 10,
            "South Africa" => 10,
            "South Georgia and the South Sandwich Islands" => 10,
            "South Sudan" => 12,
            "Spain" => 20,
            "Sri Lanka" => 12,
            "Sudan" => 18,
            "Suriname" => 10,
            "Svalbard and Jan Mayen" => 10,
            "Sweden" => 12,
            "Switzerland" => 10,
            "Syrian Arab Republic" => 10,
            "Taiwan" => 16,
            "Tajikistan" => 13,
            "Tanzania, United Republic of" => 12,
            "Thailand" => 10,
            "Togo" => 12,
            "Tokelau" => 10,
            "Tonga" => 10,
            "Trinidad and Tobago" => 10,
            "Tunisia" => 10,
            "Turkey" => 10,
            "Turkmenistan" => 16,
            "Turks and Caicos Islands" => 10,
            "Tuvalu" => 10,
            "Uganda" => 12,
            "Ukraine" => 29,
            "United Arab Emirates" => 22,
            "United Kingdom" => 8,
            "United States" => 12,
            "Uruguay" => 10,
            "Uzbekistan" => 12,
            "Vanuatu" => 10,
            "Vatican City" => 12,
            "Venezuela" => 20,
            "Viet Nam" => 10,
            "Wallis and Futuna" => 10,
            "Western Sahara" => 12,
            "Yemen" => 14,
            "Zambia" => 12,
            "Zimbabwe" => 10,
        ];

        // Check if the country exists in the formats array
        if (array_key_exists($country, $formats)) {
            $length = $formats[$country];
            return $this->generateRandomNumber($length);
        } else {
            return "Invalid country selected.";
        }
    }

    function generateAccountNumber($country) {
        // SQL query to fetch the digit length based on the country name
        $query = "SELECT caf_digit FROM countryFccountFormats WHERE caf_name = :country";

        // Prepare the SQL statement
        $stmt = $this->prepare($query);

        // Bind the country parameter to the query
        $stmt->bindParam(':country', $country);

        // Execute the query
        $stmt->execute();

        // Fetch the result
        $result = $stmt->fetch(PDO::FETCH_ASSOC);

        // Check if a record was found
        if ($result) {
            $length = $result['caf_digit'];
            // Generate the account number based on the length
            return $this->generateRandomNumber($length);
        } else {
            // Return an error if the country is not found
            return $this->generateRandomNumber('10');
        }
    }

    // Helper function to generate a random account number of a given length
    function generateRandomNumber($length) {
        // Ensure that the upper bound is calculated as an integer and not a float
        $max = (int) ('9' . str_repeat('9', $length - 1)); // Creates a number like 999... based on the length
        return str_pad(random_int(0, $max), $length, '0', STR_PAD_LEFT);
    }

    function maskAccountNumber($accountNumber) {
        // Remove any non-numeric characters (just in case)
        $cleanNumber = preg_replace('/\D/', '', $accountNumber);

        // Ensure the account number has at least 4 digits and no more than 12
//        if (strlen($cleanNumber) < 4 || strlen($cleanNumber) > 16) {
//            return "Invalid account number";
//        }

        // Get the length of the account number
        $numLength = strlen($cleanNumber);

        // Mask all digits except the last 4
        $maskedNumber = str_repeat('X', $numLength - 4) . substr($cleanNumber, -4);

        return $maskedNumber;
    }

    function checkCardStatus($expireDate, $status) {
        // Current date
        $currentDate = new DateTime();

        // Convert expire date to a DateTime object (assuming input is in 'mm/yy' format)
        $expireDateObj = DateTime::createFromFormat('m/y', $expireDate);

        // Check if card is expired
        if ($expireDateObj < $currentDate) {
            return "Expired";
        } else {
            // Check card status
            if (strtolower($status) == 'active') {
                return "Active";
            } else if(strtolower($status) =='blocked') {
                return "Blocked";
            }else{
                return "Processing";
            }
        }
    }

    function sendResetLink($email) {
        global $mailerHost;

        // Check if email exists in users table
        $stmt = $this->prepare("SELECT * FROM user_emails WHERE email = :email");
        $stmt->bindParam(':email', $email);
        $stmt->execute();
        $user = $stmt->fetch();

        if (!$user) {
            return array(
                "success" => false,
                "message" => "Email not found"
            );
        } else {
            $otp = rand(100000, 999999); // Generate a 6-digit OTP
            $otpExpire = date("Y-m-d H:i:s", strtotime('+1 hour'));

            // Update OTP and expiration in database
            $updateStmt = $this->prepare("UPDATE users SET tbl_otp = :otp, tbl_otp_expire = :otpExpire WHERE tbl_id = :id");
            $updateStmt->bindParam(':otp', $otp);
            $updateStmt->bindParam(':otpExpire', $otpExpire);
            $id = $user['user_id'];
            $updateStmt->bindParam(':id', $id);

            if ($updateStmt->execute()) {
                $resetLink = "https://" . $mailerHost . "/reset-password?email=" . urlencode($email) . "&token=" . urlencode($otp);

                // Styled HTML email content
                $mailContent = '
                <html>
                <head>
                    <style>
                        body {
                            font-family: Arial, sans-serif;
                            color: #333;
                        }
                        .container {
                            padding: 20px;
                            border: 1px solid #ddd;
                            border-radius: 8px;
                            max-width: 500px;
                            margin: 0 auto;
                            background-color: #f9f9f9;
                        }
                        .header {
                            font-size: 24px;
                            font-weight: bold;
                            color: #4CAF50;
                            margin-bottom: 20px;
                        }
                        .content {
                            font-size: 16px;
                            margin-bottom: 20px;
                        }
                        .link-button {
                            display: inline-block;
                            padding: 10px 20px;
                            background-color: #4CAF50;
                            color: #fff;
                            text-decoration: none;
                            border-radius: 5px;
                        }
                        .footer {
                            font-size: 12px;
                            color: #777;
                            margin-top: 20px;
                        }
                    </style>
                </head>
                <body>
                    <div class="container">
                        <div class="header">Password Reset Request</div>
                        <div class="content">
                            <p>We received a request to reset your password. Your OTP code is:</p>
                            <h2>' . $otp . '</h2>
                            <p>Alternatively, you can reset your password using the following link:</p>
                            <p><a href="' . $resetLink . '" class="link-button">Reset Password</a></p>
                            <p>This code will expire in 1 hour.</p>
                        </div>
                        <div class="footer">
                            <p>If you did not request a password reset, please ignore this email.</p>
                        </div>
                    </div>
                </body>
                </html>
            ';

                // Send email
                if ($this->sendEmail($email, 'Password Reset Request', $mailContent, true)) { // true for HTML email
                    return array(
                        "success" => true,
                        "message" => "Password reset link and OTP sent successfully"
                    );
                } else {
                    return array(
                        "success" => false,
                        "message" => "Failed to send password reset email"
                    );
                }
            } else {
                return array(
                    "success" => false,
                    "message" => "Failed to generate OTP"
                );
            }
        }
    }

    function verifyToken($email, $token) {
        // Prepare the SQL query with a JOIN to access `tbl_otp` and `tbl_otp_expire` in the `users` table
        $stmt = $this->prepare("
        SELECT users.tbl_otp, users.tbl_otp_expire, users.tbl_id as user_id 
        FROM users 
        INNER JOIN user_emails ON users.tbl_id = user_emails.user_id 
        WHERE user_emails.email = :email
    ");

        // Bind the email parameter
        $stmt->bindParam(':email', $email);
        $stmt->execute();
        $user = $stmt->fetch();

        // Initialize response array
        $response = [];

        if ($user) {
            // Check if the token matches and if it hasn't expired
            if ($user['tbl_otp'] == $token && strtotime($user['tbl_otp_expire']) > time()) {
                $response['success'] = true;
                $response['message'] = "Token is valid and has not expired.";
                $response['user_id'] = $user['user_id'];
            } else {
                $response['success'] = false;
                $response['message'] = "The token is invalid or has expired. Please request a new password reset.";
            }
        } else {
            $response['success'] = false;
            $response['message'] = "Invalid email address. Please check your link and try again.";
        }

        return $response;
    }

    function changeNewPassword($newPassword, $userId){
        // Hash the new password
        $hashedPassword = password_hash($newPassword, PASSWORD_DEFAULT);

        // Update the user's password in the database
        $stmt = $this->prepare("UPDATE users SET tbl_password = :password, tbl_otp = NULL, tbl_otp_expire = NULL, tbl_Last_password_changed =:last WHERE tbl_id = :id");
        $stmt->bindParam(':password', $hashedPassword);
        $stmt->bindParam(':id', $userId);
        $date = date('d M Y h:i:s a');
        $stmt->bindParam(":last", $date);

        if ($stmt->execute()) {
            $response["success"] = true;
            $response["message"] = "Password updated successfully.";
        } else {
            $response["success"] = false;
            $response["message"] = "Failed to update password. Please try again.";
        }

        return $response;
    }

    function displaySystemConfigurationMissingScreen()
    {
        echo <<<HTML
    <!DOCTYPE html><html lang="en"><head><meta charset="UTF-8"><meta name="viewport" content="width=device-width, initial-scale=1.0"><title>System Configuration Missing</title><style>*{margin:0;padding:0;box-sizing:border-box;}body,html{min-height:100vh;font-family:Arial,sans-serif;}.suspended-page{display:flex;justify-content:center;align-items:flex-start;min-height:100vh;background-color:#f1f1f1;padding-top:40px;}.suspended-content{text-align:center;background-color:#fff;padding:40px;border-radius:8px;box-shadow:0 4px 10px rgba(0,0,0,0.2);max-width:600px;width:100%;}.suspended-content h1{font-size:2.5rem;color:#dc3545;margin-bottom:20px;}.suspended-content p{font-size:1.1rem;color:#333;margin-bottom:20px;}.suspended-content .error-details{font-size:1rem;color:#666;background-color:#f8d7da;padding:15px;border-radius:5px;margin-bottom:20px;border:1px solid #dc3545;}.disclaimer{font-size:0.9rem;color:#555;text-align:left;background-color:#f0f0f0;padding:15px;border-radius:5px;border:1px solid #ccc;margin-top:20px;}</style></head><body><div class="suspended-page"><div class="suspended-content"><h1>System Configuration Missing</h1><p>It looks like some critical configuration is missing or incomplete.</p><div class="error-details"><p>Error Code: CONFIG-001</p><p>Details: The system was unable to load the necessary setup files.</p></div><p>Please consult the documentation or contact your system administrator for assistance.</p><div class="disclaimer"><p><strong>Disclaimer:</strong> This issue may arise due to missing configuration files, incorrect setup parameters, or incomplete installation. Ensure that all required files and settings are properly configured before attempting to access the system. This includes checking the database connections, server permissions, and ensuring that all environment variables are correctly set.</p><p>Any modification of system files should be done with caution. The system administrator holds the responsibility to validate the configuration prior to deployment. If this message persists, the system may be in a critical state and requires immediate attention to avoid data loss or service interruption.</p><p>By continuing to access this system without resolving the issue, you acknowledge that it may not function as intended and could cause unexpected behavior.</p></div></div></div></body></html>
HTML;
    }

    function updateProfileData($data){
        $query = 'UPDATE users SET tbl_first_name = :first_name, tbl_last_name = :last_name, tbl_state = :state, tbl_city = :city, tbl_address = :address, tbl_dob = :dob, tbl_cot = :cot, tbl_imf = :imf, tbl_transaction_pin = :transaction_pin, tbl_kyc = :kyc_status, tbl_account_status = :account_status, tbl_country = :account_country WHERE tbl_id = :user_id';
        $stmt = $this->prepare($query);

        // Bind parameters
        $stmt->bindParam(':first_name', $data['first_name']);
        $stmt->bindParam(':last_name', $data['last_name']);
        $stmt->bindParam(':state', $data['state']);
        $stmt->bindParam(':city', $data['city']);
        $stmt->bindParam(':address', $data['address']);
        $stmt->bindParam(':dob', $data['dob']);
        $stmt->bindParam(':cot', $data['cot']);
        $stmt->bindParam(':imf', $data['imf']);
        $stmt->bindParam(':transaction_pin', $data['transaction_pin']);
        $stmt->bindParam(':kyc_status', $data['kyc_status']);
        $stmt->bindParam(':account_status', $data['account_status']);
        $stmt->bindParam(':account_country', $data['account_country']);
        $stmt->bindParam(':user_id', $data['user_id']); // Replace $userId with the current user's ID
        $stmt->execute();
        return $stmt->rowCount()>0 ? 1 : 0;
    }

}
