<?php
// Include necessary initialization and database connection
require_once "profile/includes/init.php";

// Get email and token from URL parameters
$email = $_GET['email'] ?? null;
$token = $_GET['token'] ?? null;

// Verify that email and token are present
if (!$email && !$token) {

    // Missing parameters
    echo "Missing parameters. Please use the link sent to your email.";
    exit();
}


$result = $helper->verifyToken($email, $token);

if(!$result['success']){
    echo $result['message'];
    exit();
}

//print_r($result);
?>

<!DOCTYPE html>
<html lang="zxx">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="stylesheet" href="assets/css/swiper-bundle.min.css">
    <link rel="stylesheet" href="assets/css/flaticon_finto.css">
    <link rel="stylesheet" href="assets/css/scrollCue.css">
    <link rel="stylesheet" href="assets/css/remixicon.css">
    <link rel="stylesheet" href="assets/css/style.css">
    <link rel="stylesheet" href="assets/css/responsive.css">
    <title><?php echo $info['name']; ?> - Banking, & Finance</title>
    <link rel="icon" type="image/png" href="assets/images/favicon.png">
    <style>
        .spinner {
            display: none;
            width: 18px;
            height: 18px;
            border: 3px solid #ccc;
            border-top: 3px solid #333;
            border-radius: 50%;
            animation: spin 1s linear infinite;
            margin-left: 10px;
        }
        @keyframes spin {
            0% { transform: rotate(0deg); }
            100% { transform: rotate(360deg); }
        }
    </style>
</head>
<body>

<?php include "preloader.php"; ?>

<div class="forgot-password-page overflow-hidden">
    <div class="container">
        <div class="row justify-content-center">
            <div class="col-lg-6 col-md-12">
                <form class="login-form bg-color-fffaeb radius-30 border-bx">
                    <div class="text-center mb-5">
                        <a href="./" class="d-block">
                            <img src="assets/images/favicon.png" alt="image">
                        </a>
                    </div>
                    <h2 class="text-center">New Password</h2>

                    <!-- Message display area -->
                    <div id="message" class="text-center mb-4"></div>

                    <label class="form-label">New Password</label>
                    <input type="text" class="form-control" id="new-password" name="new-password" placeholder="">

                    <input type="hidden" id="user_id" name="user_id" value="<?php echo $result['user_id']; ?>"/>

                    <label class="form-label">Confirm Password</label>
                    <input type="text" class="form-control" id="confirm-password" name="confirm-password" placeholder="">

                    <!-- Button with spinner inside -->
                    <button type="button" id="changePasswordSubmit" class="default-btn w-100 text-center">
                        <span id="buttonText">Update Password</span>
                        <div class="spinner" id="buttonSpinner"></div>
                    </button>


                </form>
            </div>
        </div>
    </div>
</div>

<div class="go-top">
    <i class="ri-arrow-up-fill"></i>
</div>

<script src="assets/js/bootstrap.bundle.min.js"></script>
<script src="assets/js/swiper-bundle.min.js"></script>
<script src="assets/js/fslightbox.min.js"></script>
<script src="assets/js/smooth-scroll.js"></script>
<script src="assets/js/scrollCue.min.js"></script>
<script src="assets/js/script.js"></script>
<script>
    document.getElementById('changePasswordSubmit').addEventListener('click', function () {
        const newPassword = document.getElementById('new-password').value;
        const confirmPassword = document.getElementById('confirm-password').value;
        const userId = document.getElementById('user_id').value;

        const messageDiv = document.getElementById('message');
        const buttonText = document.getElementById('buttonText');
        const buttonSpinner = document.getElementById('buttonSpinner');

        messageDiv.innerHTML = '';

        buttonText.style.display = 'none';
        buttonSpinner.style.display = 'inline-block';

        //alert(newPassword)

        fetch('profile/process/change-password.php', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({ newPassword: newPassword,confirmPassword:confirmPassword, user_id: userId })
        })
            .then(response => response.json())
            .then(data => {

                if (data.success) {

                    alert(data.message)

                    window.location.href='login'

                } else {
                    messageDiv.innerHTML = '<p style="color: red;">Passwords do not match.</p>';
                    buttonText.style.display = 'inline';
                    buttonSpinner.style.display = 'none';

                }
            })
            .catch(error => {

                console.error("Error:", error);
                messageDiv.innerHTML = '<p style="color: red;">An error occurred. Please try again.</p>';
                buttonText.style.display = 'inline';
                buttonSpinner.style.display = 'none';

            });

    })
</script>

</body>
</html>
